
##########################################################
##########################################################
##########################################################
#
#       BCG program
#
#       Analysis of     non-vaccinated (non-BCG) and
#                       vaccinated (BCG) data
#
#       Input data in   infile0
#                       infile1
#
##########################################################
##########################################################
##########################################################

Version <- "BCG (version 3.1, December 2004)"

obj <- objects()





bcg <- function()
{

if (is.na(match("RndSeed",obj))) RndSeed <- runif(1,0,100)

set.seed(RndSeed)

if (is.na(match("outfile",obj))) outfile <- "TBMixOut"

outfilelog <- paste(outfile,".log",sep="")
outfile1 <- paste(outfile,"1.txt",sep="")
outfile2 <- paste(outfile,"2.txt",sep="")
outfile3 <- paste(outfile,"3.txt",sep="")

############################################################
sink(file=outfilelog,append=T)
 cat("\n",paste("Program",Version),"\n")
 cat(date(),"\n")
sink()
Sys.sleep(1)
############################################################


############################################################
sink(file=outfilelog,append=T)
 cat("Preparatory steps","\n")
 cat(" ...Load and process data","\n")
sink()
Sys.sleep(1)
############################################################

if (is.na(match("header",obj))) header <- F

num.cova <-  length(freq0.column);
ds0 <- read.table(file=infile0,header=header)
ds0 <- as.matrix(ds0)
ds1 <- read.table(file=infile1,header=header)
ds1 <- as.matrix(ds1)

if ( dim(ds0)[1]==1 & dim(ds0)[2]==31 )
{
warning("Frequencies should be entered as column data!")
ds0 <- t(ds0)
}


if ( dim(ds1)[1]==1 & dim(ds1)[2]==31 )
{
warning("Frequencies should be entered as column data!")
ds1 <- t(ds1)
}

if (is.na(match("freq0.column",obj)))   freq0.column <- seq(1,dim(ds0)[2])
if (is.na(match("freq1.column",obj)))   freq1.column <- seq(1,dim(ds1)[2])

if ( length(freq0.column)>1  & is.na(match("group.names",obj)))    group.names <- paste("Group",seq(1,length(freq0.column)))
if ( length(freq0.column)==1 & is.na(match("group.names",obj)))    group.names <- ""


ind.num <- dim(ds0)[1]-1
ind <- seq(1,ind.num);
freq0zero <- ds0[,freq0.column];
freq0 <-  ds0[2:(ind.num+1),freq0.column];
freq1zero <- ds1[,freq1.column];
freq1 <-  ds1[2:(ind.num+1),freq1.column];



if (is.na(match("ind.brk",obj)))        {
ind.brk <- seq(1.5,by=1,length=dim(ds0)[1]-2)
}

if (length(freq0.column)==1)     {
freq0zero <- matrix(freq0zero,ind.num+1,1)
freq0  <- matrix(freq0 ,ind.num,1)
}

if (length(freq1.column)==1)     {
freq1zero <- matrix(freq1zero,ind.num+1,1)
freq1  <- matrix(freq1 ,ind.num,1)
}

if (length(freq0.column)>1)      {
n0.total <- apply(freq0zero,2,sum)
}
else    {
n0.total <- sum(freq0zero)
}

if (length(freq1.column)>1)      {
n1.total <- apply(freq1zero,2,sum)
}
else    {
n1.total <- sum(freq1zero)
}

if (num.cova>1)
freq0all <- rbind(freq0zero,n0.total);
if (num.cova>1)
freq1all <- rbind(freq1zero,n1.total);

if (num.cova==1)
freq0all <- matrix(c(freq0zero,n0.total),ncol=1);
if (num.cova==1)
freq1all <- matrix(c(freq1zero,n1.total),ncol=1);

if (ind.num==30)
pred.int <- matrix(c(1,2,3,7,8,12,13,17,18,22,23,27,28,30),7,2,byrow=T);

if (is.na(match("ind.names",obj)))      {

if (ind.num==30)  ind.names <- paste(seq(1,30),"mm",sep="")
else ind.names <- paste("ind-",seq(1,ind.num),sep="")
ind0 <- ifelse(ind.num==30,"0mm","ind-0")
}

ind.names0 <- c(ind0,ind.names)
ind.namesntot <- c(ind.names0,"total")

if (ind.num==30) ind.brk <- seq(1.5,29.5,by=1);

if (ind.num==30)        {
len1 <- dim(pred.int)[1];
freq01 <- matrix(0,len1,num.cova);
freq11 <- matrix(0,len1,num.cova);
ind.names1 <- character(0);
for (jj in 1:len1)
ind.names1[jj] <- paste(pred.int[jj,1], "to", pred.int[jj,2], "mm");
}

for (j in 1:num.cova)   {
if (ind.num==30)        {
for (jj in 1:len1)
freq01[jj,j] <- sum( freq0[pred.int[jj,1]:pred.int[jj,2],j]);
}
}


for (j in 1:num.cova)   {
if (ind.num==30)        {
for (jj in 1:len1)
freq11[jj,j] <- sum( freq1[pred.int[jj,1]:pred.int[jj,2],j]);
}
}

#       normalizing likelihood (will be subtracted from likelihood)
l0 <- sum( apply(freq0zero,2,function(x) sum(x*log(x/sum(x)),na.rm=T) ) )
l1 <- sum( apply(freq1zero,2,function(x) sum(x*log(x/sum(x)),na.rm=T) ) )
LLnorm <- l0+l1


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Set tuning parameters for Metropolis Sampler","\n")
sink()
Sys.sleep(1)
############################################################


if (is.na(match("MS.run",obj))) MS.run <- T
if (is.na(match("MS.results",obj))) MS.results <- T
if (is.na(match("MS.check",obj))) MS.check <- F
if (is.na(match("MS.graph",obj))) MS.graph <- c(T,F,F,F,T)

if (is.na(match("MS.graph.page",obj))) {
pg <- ceiling(sqrt(num.cova))
if (pg*(pg-1)>=num.cova) MS.graph.page <- c(pg-1,pg) else MS.graph.page <- c(pg,pg)
}

if (is.na(match("MS.size",obj))) MS.size <- c(length(freq0.column)*500,5000,1)
if (length(MS.size)==2) MS.size[3] <- 1

if (is.na(match("MS.screen",obj))) MS.screen <- c(11,13, 1, 3, 5, 7, 9 )
if (is.na(match("MS.screen.dig",obj))) MS.screen.dig <- 3

if (is.na(match("MS.adj",obj))) MS.adj <- 200


MS.names <- c(
"TB0.qnt1",
"TB0.qnt2",
"CR0.qnt1",
"CR0.qnt2",
"TB1.qnt1",
"TB1.qnt2",
"CR1.qnt1",
"CR1.qnt2",
"BCG.qnt1",
"BCG.qnt2",
"p0.TB",
"p0.zero",
"p1.TB",
"p1.zero",
"p1.BCG")

############################################################
sink(file=outfilelog,append=T)
cat(" ...Load information from former analysis","\n")
sink()
Sys.sleep(1)
############################################################

if (!is.na(match("TBmix",obj))) {
fit <- TBmix$fit
pred0.table <- TBmix$pred0.table
p0.TB.summary <- TBmix$p0.TB.summary
p0.zero.summary <- TBmix$p0.zero.summary
TB0.qnt1.summary <- TBmix$TB0.qnt1.summary
TB0.qnt2.summary <- TBmix$TB0.qnt2.summary
CR0.qnt1.summary <- TBmix$CR0.qnt1.summary
CR0.qnt2.summary <- TBmix$CR0.qnt2.summary
pred1.table <- TBmix$pred1.table
p1.TB.summary <- TBmix$p1.TB.summary
p1.zero.summary <- TBmix$p1.zero.summary
TB1.qnt1.summary <- TBmix$TB1.qnt1.summary
TB1.qnt2.summary <- TBmix$TB1.qnt2.summary
CR1.qnt1.summary <- TBmix$CR1.qnt1.summary
CR1.qnt2.summary <- TBmix$CR1.qnt2.summary
BCG.qnt1.summary <- TBmix$BCG.qnt1.summary
BCG.qnt2.summary <- TBmix$BCG.qnt2.summary
mpe0 <- TBmix$mpe
LastValues <- TBmix$LastValues
}
else    {
fit <- NA
pred0.table <- NA
p0.TB.summary <- NA
p0.zero.summary <-  NA
TB0.qnt1.summary <- NA
TB0.qnt2.summary <- NA
CR0.qnt1.summary <- NA
CR0.qnt2.summary <- NA
pred1.table <- NA
p1.TB.summary <- NA
p1.zero.summary <- NA
TB1.qnt1.summary <- NA
TB1.qnt2.summary <- NA
CR1.qnt1.summary <- NA
CR1.qnt2.summary <- NA
BCG.qnt1.summary <- NA
BCG.qnt2.summary <- NA
mpe0 <- NA
}

############################################################
sink(file=outfilelog,append=T)
cat(" ...Set values for parameter constraints","\n")
sink()
Sys.sleep(1)
############################################################

# intervals
if (is.na(match("TB0.qnt1.int",obj)))  TB0.qnt1.int <- c(0,40)
if (is.na(match("TB0.qnt2.int",obj)))  TB0.qnt2.int <- c(0,40)
if (is.na(match("CR0.qnt1.int",obj)))  CR0.qnt1.int <- c(0,40)
if (is.na(match("CR0.qnt2.int",obj)))  CR0.qnt2.int <- c(0,40)

if (is.na(match("TB1.qnt1.int",obj)))  TB1.qnt1.int <- c(0,40)
if (is.na(match("TB1.qnt2.int",obj)))  TB1.qnt2.int <- c(0,40)
if (is.na(match("CR1.qnt1.int",obj)))  CR1.qnt1.int <- c(0,40)
if (is.na(match("CR1.qnt2.int",obj)))  CR1.qnt2.int <- c(0,40)

if (is.na(match("BCG.qnt1.int",obj)))  BCG.qnt1.int <- c(0,40)
if (is.na(match("BCG.qnt2.int",obj)))  BCG.qnt2.int <- c(0,40)

if (is.na(match("p0.TB.int",obj)))          p0.TB.int <-  c(0,1)
if (is.na(match("p0.zero.int",obj)))    p0.zero.int <-  c(0,1)
if (is.na(match("p1.TB.int",obj)))          p1.TB.int <-  c(0,1)
if (is.na(match("p1.zero.int",obj)))    p1.zero.int <-  c(0,1)
if (is.na(match("p1.BCG.int",obj)))     p1.BCG.int <-  c(0,1)

# diff
if (is.na(match("TB0.qnt1.diff",obj))) TB0.qnt1.diff <- 0
if (is.na(match("TB0.qnt2.diff",obj))) TB0.qnt2.diff <- 0
if (is.na(match("CR0.qnt1.diff",obj))) CR0.qnt1.diff <- 0
if (is.na(match("CR0.qnt2.diff",obj))) CR0.qnt2.diff <- 0

if (is.na(match("TB1.qnt1.diff",obj))) TB1.qnt1.diff <- 0
if (is.na(match("TB1.qnt2.diff",obj))) TB1.qnt2.diff <- 0
if (is.na(match("CR1.qnt1.diff",obj))) CR1.qnt1.diff <- 0
if (is.na(match("CR1.qnt2.diff",obj))) CR1.qnt2.diff <- 0

if (is.na(match("BCG.qnt1.diff",obj))) BCG.qnt1.diff <- 0
if (is.na(match("BCG.qnt2.diff",obj))) BCG.qnt2.diff <- 0

# ratio
if (is.na(match("TB0.qnt1.ratio",obj))) TB0.qnt1.ratio <- 1
if (is.na(match("TB0.qnt2.ratio",obj))) TB0.qnt2.ratio <- 1
if (is.na(match("CR0.qnt1.ratio",obj))) CR0.qnt1.ratio <- 1
if (is.na(match("CR0.qnt2.ratio",obj))) CR0.qnt2.ratio <- 1

if (is.na(match("TB1.qnt1.ratio",obj))) TB1.qnt1.ratio <- 1
if (is.na(match("TB1.qnt2.ratio",obj))) TB1.qnt2.ratio <- 1
if (is.na(match("CR1.qnt1.ratio",obj))) CR1.qnt1.ratio <- 1
if (is.na(match("CR1.qnt2.ratio",obj))) CR1.qnt2.ratio <- 1

if (is.na(match("BCG.qnt1.ratio",obj))) BCG.qnt1.ratio <- 1
if (is.na(match("BCG.qnt2.ratio",obj))) BCG.qnt2.ratio <- 1


#  assumptions on differences for TB and CR distributions (non-BCG vs BCG)

if (is.na(match("TB01.qnt1.diff",obj))) TB01.qnt1.diff <- 0
if (is.na(match("TB01.qnt2.diff",obj))) TB01.qnt2.diff <- 0
if (is.na(match("CR01.qnt1.diff",obj))) CR01.qnt1.diff <- 0
if (is.na(match("CR01.qnt2.diff",obj))) CR01.qnt2.diff <- 0

if (is.na(match("TB01.qnt1.ratio",obj))) TB01.qnt1.ratio <- 1
if (is.na(match("TB01.qnt2.ratio",obj))) TB01.qnt2.ratio <- 1
if (is.na(match("CR01.qnt1.ratio",obj))) CR01.qnt1.ratio <- 1
if (is.na(match("CR01.qnt2.ratio",obj))) CR01.qnt2.ratio <- 1

if ( all(unlist(TB01.qnt1.diff)==0)) TBcc1a <- T else TBcc1a <- F
if ( all(unlist(TB01.qnt2.diff)==0)) TBcc2a <- T else TBcc2a <- F
if ( all(unlist(CR01.qnt1.diff)==0)) CRcc1a <- T else CRcc1a <- F
if ( all(unlist(CR01.qnt2.diff)==0)) CRcc2a <- T else CRcc2a <- F

if ( all(unlist(TB01.qnt1.ratio)==1)) TBcc1b <- T else TBcc1b <- F
if ( all(unlist(TB01.qnt2.ratio)==1)) TBcc2b <- T else TBcc2b <- F
if ( all(unlist(CR01.qnt1.ratio)==1)) CRcc1b <- T else CRcc1b <- F
if ( all(unlist(CR01.qnt2.ratio)==1)) CRcc2b <- T else CRcc2b <- F

TBcc1 <- (TBcc1a & TBcc1b)
TBcc2 <- (TBcc2a & TBcc2b)
CRcc1 <- (CRcc1a & CRcc1b)
CRcc2 <- (CRcc2a & CRcc2b)


if (is.na(match("qnt1TB",obj))) qnt1TB <- 0.50
if (is.na(match("qnt2TB",obj))) qnt2TB <- 0.95
if (is.na(match("qnt1CR",obj))) qnt1CR <- 0.50
if (is.na(match("qnt2CR",obj))) qnt2CR <- 0.95
if (is.na(match("qnt1BCG",obj))) qnt1BCG <- 0.50
if (is.na(match("qnt2BCG",obj))) qnt2BCG <- 0.95

TB0.x1 <- qnorm(qnt1TB);
TB0.x2 <- qnorm(qnt2TB);
CR0.x1 <- qnorm(qnt1CR);
CR0.x2 <- qnorm(qnt2CR);
TB1.x1 <- qnorm(qnt1TB);
TB1.x2 <- qnorm(qnt2TB);
CR1.x1 <- qnorm(qnt1CR);
CR1.x2 <- qnorm(qnt2CR);
BCG.x1 <- qnorm(qnt1BCG);
BCG.x2 <- qnorm(qnt2BCG);


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Load functions","\n")
sink()
Sys.sleep(1)
############################################################

#       MakeGroupNames
#       MakeGroupIndices
#       MakeGroupSelect
#       CompareGroupsDiff
#       CompareGroupsRatio
#       MakeCheck
#       PrepareCheck
#       Checks
#       loglik0 (non-BCG)
#       loglik1 (BCG)
#       loglik (both)
#       prior0
#       trans0
#       mixture0 (non-BCG)
#       mixture1 (BCG)

MakeGroupNames <- function(group.names)
{
groups.vector <- group.names[[1]]
if (length(group.names)>1)  {
for (j in seq(2,length(group.names)) ) {
groups.vector <- as.vector(t( outer(groups.vector,group.names[[j]],paste) ))
}
}
return(group.names=groups.vector)
}

MakeGroupIndices <- function(groups.indices)
{
groups.vector <- groups.indices[[length(groups.indices)]]
for (j in ( (length(groups.indices)-1)):1)       {
groups.vector <-  rep((groups.indices[[j]]-1)*length(groups.vector),each=length(groups.vector))+groups.vector
}
}

MakeGroupSelect <- function(groups.sizes)
{
dlist <- list()

for (j in 1:length(groups.sizes))
{
dlist[[j]] <- seq(1,groups.sizes[j])
}

groups.select <- matrix(NA,prod(groups.sizes),length(groups.sizes) )

for (i in 1:length(groups.sizes))
{
for (j in 1:groups.sizes[i])
{
ddlist <- dlist
ddlist[[i]] <- rep(j,groups.sizes[i])
mmm <- MakeGroupIndices( ddlist)
groups.select[mmm,i] <- j
}
}
return(groups.select)
}

CompareGroupsDiff <- function(groups.select,groups.sizes,crit)
{
sss <- seq(1,prod(groups.sizes))
Alist <- list()

for (k in 1:length(groups.sizes))
        {

if (any(crit[[k]] !=0))
                {
A <- numeric(0)

for (i in 1:(groups.sizes[k]-1) )
{
if (length(crit[[k]])==1)
 jup <- groups.sizes[k]
 else
 jup <- i+1
for (j in (i+1):jup   )
{

e1 <- groups.select[,k]==i
e2 <- groups.select[,k]==j
AA1 <- sss[e1]
AA2 <- sss[e2]
cr <- unlist(crit[[k]])
AA <- cbind(AA1,AA2,matrix(cr,sum(e1),length(cr),byrow=T))
A <- rbind(A,AA)
}
}
Alist[[k]] <- A
}
else Alist[[k]] <- NA
}

return(Alist)
}

CompareGroupsRatio <- function(groups.select,groups.sizes,crit)
{
sss <- seq(1,prod(groups.sizes))

Alist <- list()

for (k in 1:length(groups.sizes))
        {


if ( any(crit[[k]] !=1))
                {
A <- numeric(0)

for (i in 1:(groups.sizes[k]-1) )
{
if (length(crit[[k]])==1)
 jup <- groups.sizes[k]
 else
 jup <- i+1
for (j in (i+1):jup   )
{

e1 <- groups.select[,k]==i
e2 <- groups.select[,k]==j
AA1 <- sss[e1]
AA2 <- sss[e2]
cr <- unlist(crit[[k]])
AA <- cbind(AA1,AA2,matrix(cr,sum(e1),length(cr),byrow=T))
A <- rbind(A,AA)
}
}
Alist[[k]] <- A
}
else Alist[[k]] <- NA

}
return(Alist)
}

MakeCheck <- function(para,CG,comp)
{

if (!is.list(CG)) CG <- list(CG)

TFvec <- rep(NA,length(CG))

for (j in 1:length(CG))
{
CG1 <- CG[[j]]

if ( all(is.na(CG1)) ) TF <- T
else
{

if (comp=="diff")
{
out <- para[CG1[,2]]-para[CG1[,1]]
if (ncol(CG1)==3) TF <- abs(out) < abs(CG1[,3])
if (ncol(CG1)==4) TF <- out < CG1[,4] & out > CG1[,3]
}

if (comp=="ratio")
{
out <- para[CG1[,2]]/para[CG1[,1]]
if (ncol(CG1)==3) TF <- abs(log(out)) < log(CG1[,3])
if (ncol(CG1)==4) TF <- out < CG1[,4] & out > CG1[,3]
}

TF <- all(TF)
}

TFvec[j] <- TF

}
return(all(TFvec))
}


PrepareCheck <- function(group.names,crit,comp)
{

if (is.list(group.names))
{
groups.sizes <- unlist(lapply(group.names[2:length(group.names)],length))

if (length(crit)>1)
 {
 GS <- MakeGroupSelect(groups.sizes)

 if (comp=="diff") rt <- list(Check=any(unlist(crit)!=0),comp,CheckValues=CompareGroupsDiff(GS,groups.sizes,crit))
 if (comp=="ratio") rt <- list(Check=any(unlist(crit)!=1),comp,CheckValues=CompareGroupsRatio(GS,groups.sizes,crit))
 }

if (length(crit)==1)
{
 if (crit==0 & comp=="diff"   ) rt <- list(Check=F,comp,CheckValues=0)
 if (crit>0  & comp=="diff"    ) rt <- list(Check=T,comp,CheckValues=crit)
 if (crit==1 & comp=="ratio"   ) rt <- list(Check=F,comp,CheckValues=1)
 if (crit>1  & comp=="ratio"    ) rt <- list(Check=T,comp,CheckValues=crit)
}
}  # end of list

else
{
groups.sizes <- num.cova
if (length(group.names)==1)
        rt <- list(Check=F,CheckValues=0)

if (length(group.names)>1 )
{
if (length(crit)==1)
{
 if (crit==0 & comp=="diff"   ) rt <- list(Check=F,comp,CheckValues=0)
 if (crit>0  & comp=="diff"    ) rt <- list(Check=T,comp,CheckValues=crit)
 if (crit==1 & comp=="ratio"   ) rt <- list(Check=F,comp,CheckValues=1)
 if (crit>1  & comp=="ratio"    ) rt <- list(Check=T,comp,CheckValues=crit)
}
if (length(crit)>1)
 {
 #GS <- matrix(seq(1:num.cova),num.cova,1)
 #if (comp=="diff") rt <- list(Check=any(unlist(crit)!=0),comp,CheckValues=CompareGroupsDiff(GS,groups.sizes,crit))
 #if (comp=="ratio") rt <- list(Check=any(unlist(crit)!=1),comp,CheckValues=CompareGroupsRatio(GS,groups.sizes,crit))
 rt <- list( Check=T, comp,
 CheckValues=cbind( seq(1:(num.cova-1)), seq(2,num.cova), rep(crit[1],num.cova-1), rep(crit[2],num.cova-1) ) )
 }
}
}       # end of vector

return(rt)
}


Checks <- function(para,PC)
{
if (!PC[[1]])
        rt <- T

if (PC[[1]])
{

if (length(PC[[3]])==1)
{
 if (PC[[2]]=="diff")
 mm <- max(para)-min(para)
 if (PC[[2]]=="ratio")
 mm <- max(para)/min(para)
 rt <- mm<=PC[[3]]
}

if (length(PC[[3]])>1)
{
 rt <- MakeCheck(para,PC[[3]],PC[[2]])
}
}
return(rt)
}



#       FUNCTIONS:  LOG-LIKELIHOOD, LOG-PRIOR

loglik0 <- function(theta,x)
{
theta[,1] <- theta[groups.indices[,1],1]
theta[,2] <- theta[groups.indices[,2],2]
theta[,3] <- theta[groups.indices[,3],3]
theta[,4] <- theta[groups.indices[,4],4]

p1      <- theta[,11];
p0      <- theta[,12];
TB0.qnt1  <- theta[,1];
TB0.qnt2  <- theta[,2];
CR0.qnt1  <- theta[,3];
CR0.qnt2  <- theta[,4];

ll <- -1e300;

cond2a <- Checks(TB0.qnt1,TB0.qnt1.diff.PC)
cond3a <- Checks(TB0.qnt2,TB0.qnt2.diff.PC)
cond4a <- Checks(CR0.qnt1,CR0.qnt1.diff.PC)
cond5a <- Checks(CR0.qnt2,CR0.qnt2.diff.PC)

cond2b <- Checks(TB0.qnt1,TB0.qnt1.ratio.PC)
cond3b <- Checks(TB0.qnt2,TB0.qnt2.ratio.PC)
cond4b <- Checks(CR0.qnt1,CR0.qnt1.ratio.PC)
cond5b <- Checks(CR0.qnt2,CR0.qnt2.ratio.PC)

cond10 <- all(TB0.qnt1<TB0.qnt2)
cond11 <- all(CR0.qnt1<CR0.qnt2)
cond12 <- all(CR0.qnt1<TB0.qnt1)
cond13 <- all(CR0.qnt2<TB0.qnt2)
cond20 <- all(p1+p0<1)


cond <- c(cond2a,cond3a,cond4a,cond5a,cond2b,cond3b,cond4b,cond5b, cond10,cond11,cond12,cond13,cond20)
if ( all(cond) )      {
ll <- 0
for (jjj in 1:num.cova) {
if (distTB=="Wb")
 dist1 <- Wb.dsc(ind.brk,TB0.qnt1[jjj],TB0.qnt2[jjj],c(qnt1TB,qnt2TB) );
if (distTB=="LN")       {
  sigma  <- (log(TB0.qnt2[jjj])-log(TB0.qnt1[jjj]))/(TB0.x2-TB0.x1);
  mu <- log(TB0.qnt1[jjj])-sigma*TB0.x1;
  dist1 <- LN.dsc(ind.brk,mu,sigma,0);
}
if (distTB=="N")        {
  sigma  <- (TB0.qnt2[jjj]-TB0.qnt1[jjj])/(TB0.x2-TB0.x1);
  mu <- TB0.qnt1[jjj]-sigma*TB0.x1;
  dist1 <- N.dsc(ind.brk,mu,sigma);
}
if (distCR=="Wb")
  dist2 <- Wb.dsc(ind.brk,CR0.qnt1[jjj],CR0.qnt2[jjj],c(qnt1CR,qnt2CR) );
if (distCR=="LN")       {
  sigma   <- (log(CR0.qnt2[jjj])-log(CR0.qnt1[jjj]))/(CR0.x2-CR0.x1);
  mu <- log(CR0.qnt1[jjj])-sigma*CR0.x1;
  dist2 <- LN.dsc(ind.brk,mu,sigma,0);
}

dist1[is.na(dist1)] <- 1e-300;
dist2[is.na(dist2)] <- 1e-300;
lladd <- freq0[,jjj]*log( (1-p0[jjj]-p1[jjj])*dist2 + p1[jjj]*dist1) ;

if(any(is.na(lladd))) lladd <- -1e300;
ll <- ll+sum(lladd);
}
}
ll <- ll+sum(freq0zero[1,]*log(p0));
return(ll);
}


loglik1 <- function(theta,x)
{

theta[,1] <- theta[groups.indices[,1],1]
theta[,2] <- theta[groups.indices[,2],2]
theta[,3] <- theta[groups.indices[,3],3]
theta[,4] <- theta[groups.indices[,4],4]

theta[,5] <- theta[groups.indices[,5],5]
theta[,6] <- theta[groups.indices[,6],6]
theta[,7] <- theta[groups.indices[,7],7]
theta[,8] <- theta[groups.indices[,8],8]
theta[,9] <- theta[groups.indices[,9],9]
theta[,10] <- theta[groups.indices[,10],10]


if (TBcc1) theta[,5] <- theta[,1]
if (TBcc2) theta[,6] <- theta[,2]
if (CRcc1) theta[,7] <- theta[,3]
if (CRcc2) theta[,8] <- theta[,4]

p1      <- theta[,13]
p0      <- theta[,14]
p3      <- theta[,15]
p2 <- 1-p1-p3-p0

TB1.qnt1  <- theta[,5];
TB1.qnt2  <- theta[,6];
CR1.qnt1  <- theta[,7];
CR1.qnt2  <- theta[,8];
BCG.qnt1  <- theta[,9];
BCG.qnt2  <- theta[,10];



ll <- -1e300;

cond5a <- Checks(TB1.qnt1,TB1.qnt1.diff.PC)
cond6a <- Checks(TB1.qnt2,TB1.qnt2.diff.PC)
cond7a <- Checks(CR1.qnt1,CR1.qnt1.diff.PC)
cond8a <- Checks(CR1.qnt2,CR1.qnt2.diff.PC)

cond9a <- Checks(BCG.qnt1,BCG.qnt1.diff.PC)
cond10a <- Checks(BCG.qnt2,BCG.qnt2.diff.PC)

cond5b <- Checks(TB1.qnt1,TB1.qnt1.ratio.PC)
cond6b <- Checks(TB1.qnt2,TB1.qnt2.ratio.PC)
cond7b <- Checks(CR1.qnt1,CR1.qnt1.ratio.PC)
cond8b <- Checks(CR1.qnt2,CR1.qnt2.ratio.PC)

cond9b <- Checks(BCG.qnt1,BCG.qnt1.ratio.PC)
cond10b <- Checks(BCG.qnt2,BCG.qnt2.ratio.PC)


cc1a <- T
if (!TBcc1a)
if ( length(TB01.qnt1.diff)==1 )
     cc1a <- all( abs(theta[,5]-theta[,1]) < TB01.qnt1.diff )
 else
     cc1a <- all( (theta[,5]-theta[,1]) > TB01.qnt1.diff[1] & (theta[,5]-theta[,1]) < TB01.qnt1.diff[2] )


cc2a <- T
if (!TBcc2a)
if ( length(TB01.qnt2.diff)==1 )
     cc2a <- all( abs(theta[,6]-theta[,2]) <TB01.qnt2.diff )
 else
     cc2a <- all( (theta[,6]-theta[,2]) > TB01.qnt2.diff[1] & (theta[,6]-theta[,2]) < TB01.qnt2.diff[2] )


cc3a <- T
if (!CRcc1a)
if ( length(CR01.qnt1.diff)==1 )
     cc3a <- all( abs(theta[,7]-theta[,3]) < CR01.qnt1.diff )
 else
     cc3a <- all( (theta[,7]-theta[,3]) > CR01.qnt1.diff[1] & (theta[,7]-theta[,3]) < CR01.qnt1.diff[2] )


cc4a <- T
if (!CRcc2a)
if ( length(CR01.qnt2.diff)==1 )
     cc4a <- all( abs(theta[,8]-theta[,4]) < CR01.qnt2.diff )
 else
     cc4a <- all( (theta[,8]-theta[,4]) > CR01.qnt2.diff[1] & (theta[,8]-theta[,4]) < CR01.qnt2.diff[2] )

cc1b <- T
if (!TBcc1b)
if ( length(TB01.qnt1.ratio)==1 )
     cc1b <- all( abs(log(theta[,5]/theta[,1])) < log(TB01.qnt1.ratio) )
 else
     cc1b <- all( (theta[,5]/theta[,1]) > TB01.qnt1.ratio[1] & (theta[,5]/theta[,1]) < TB01.qnt1.ratio[2] )

cc2b <- T
if (!TBcc2b)
if ( length(TB01.qnt2.ratio)==1 )
     cc2b <- all( abs(log(theta[,6]/theta[,2])) < log(TB01.qnt2.ratio) )
 else
     cc2b <- all( (theta[,6]/theta[,2]) > TB01.qnt2.ratio[1] & (theta[,6]/theta[,2]) < TB01.qnt2.ratio[2] )


cc3b <- T
if (!CRcc1b)
if ( length(CR01.qnt1.ratio)==1 )
     cc3b <- all( abs(log(theta[,7]/theta[,3])) < log(CR01.qnt1.ratio) )
 else
     cc3b <- all( (theta[,7]/theta[,3]) > CR01.qnt1.ratio[1] & (theta[,7]/theta[,3]) < CR01.qnt1.ratio[2] )

cc4b <- T
if (!CRcc2b)
if ( length(CR01.qnt2.ratio)==1 )
     cc4b <- all( abs(log(theta[,8]/theta[,4])) < log(CR01.qnt2.ratio) )
 else
     cc4b <- all( (theta[,8]/theta[,4]) > CR01.qnt2.ratio[1] & (theta[,8]/theta[,4]) < CR01.qnt2.ratio[2] )



cond20 <- all(TB1.qnt1<TB1.qnt2)
cond21 <- all(CR1.qnt1<CR1.qnt2)

cond22 <- all(CR1.qnt1<TB1.qnt1)
cond23 <- all(CR1.qnt2<TB1.qnt2)

cond30 <- all(BCG.qnt1<BCG.qnt2)
cond31 <- all(BCG.qnt1<TB1.qnt1)
cond32 <- all(BCG.qnt2<TB1.qnt2)

cond33 <- all(BCG.qnt1>CR1.qnt1)
cond34 <- all(BCG.qnt2>CR1.qnt2)

cond40 <- all(p1+p0+p3<1)

cond <- c(
cond5a,cond6a,cond7a,cond8a,cond9a,cond10a,
cond5b,cond6b,cond7b,cond8b,cond9b,cond10b,
cc1a,cc2a,cc3a,cc4a,cc1b,cc2b,cc3b,cc4b,
cond20,cond21,cond22,cond23,
cond30,cond31,cond32,cond33,cond34,
cond40
)


if ( all(cond) )

     {


ll <- 0
for (jjj in 1:num.cova) {

if (distTB=="Wb")
 dist1 <- Wb.dsc(ind.brk,TB1.qnt1[jjj],TB1.qnt2[jjj],c(qnt1TB,qnt2TB) );

if (distTB=="LN")       {
  sigma  <- (log(TB1.qnt2[jjj])-log(TB1.qnt1[jjj]))/(TB1.x2-TB1.x1);
  mu <- log(TB1.qnt1[jjj])-sigma*TB1.x1;
  dist1 <- LN.dsc(ind.brk,mu,sigma,0);
}

if (distTB=="N")        {
  sigma  <- (TB1.qnt2[jjj]-TB1.qnt1[jjj])/(TB1.x2-TB1.x1);
  mu <- TB1.qnt1[jjj]-sigma*TB1.x1;
  dist1 <- N.dsc(ind.brk,mu,sigma);
}

if (distCR=="Wb")
  dist2 <- Wb.dsc(ind.brk,CR1.qnt1[jjj],CR1.qnt2[jjj],c(qnt1CR,qnt2CR) );

if (distCR=="LN")       {
  sigma   <- (log(CR1.qnt2[jjj])-log(CR1.qnt1[jjj]))/(CR1.x2-CR1.x1);
  mu <- log(CR1.qnt1[jjj])-sigma*CR1.x1;
  dist2 <- LN.dsc(ind.brk,mu,sigma,0);
}

if (distBCG=="Wb")
 dist3 <- Wb.dsc(ind.brk,BCG.qnt1[jjj],BCG.qnt2[jjj],c(qnt1BCG,qnt2BCG) );

if (distBCG=="LN")       {
  sigma  <- (log(BCG.qnt2[jjj])-log(BCG.qnt1[jjj]))/(BCG.x2-BCG.x1);
  mu <- log(BCG.qnt1[jjj])-sigma*BCG.x1;
  dist3 <- LN.dsc(ind.brk,mu,sigma,0);
}

if (distBCG=="N")        {
  sigma  <- (BCG.qnt2[jjj]-BCG.qnt1[jjj])/(BCG.x2-BCG.x1);
  mu <- BCG.qnt1[jjj]-sigma*BCG.x1;
  dist3 <- N.dsc(ind.brk,mu,sigma);
}


dist1[is.na(dist1)] <- 1e-300;
dist2[is.na(dist2)] <- 1e-300;
dist3[is.na(dist3)] <- 1e-300;

lladd <- freq1[,jjj]*log( p1[jjj]*dist1 + p2[jjj]*dist2 + p3[jjj]*dist3) ;

if(any(is.na(lladd))) lladd <- -1e300;
ll <- ll+sum(lladd);
}
}
ll <- ll+sum(freq1zero[1,]*log(p0));

return(ll);
}


loglik <- function(theta,x)
{
l0 <- loglik0(theta,x)
l1 <- loglik1(theta,x)
#print(c(l0,l1))
return( l0+l1-LLnorm)
}

prior0 <- function(theta)       {return(0)}

trans0 <- function(theta,x)
{
theta[,1] <- theta[groups.indices[,1],1]
theta[,2] <- theta[groups.indices[,2],2]
theta[,3] <- theta[groups.indices[,3],3]
theta[,4] <- theta[groups.indices[,4],4]
theta[,5] <- theta[groups.indices[,5],5]
theta[,6] <- theta[groups.indices[,6],6]
theta[,7] <- theta[groups.indices[,7],7]
theta[,8] <- theta[groups.indices[,8],8]
theta[,9] <- theta[groups.indices[,9],9]
theta[,10] <- theta[groups.indices[,10],10]

if (TBcc1) TB.qnt1 <- theta[,1] else TB.qnt1 <- theta[,5]
if (TBcc2) TB.qnt2 <- theta[,2] else TB.qnt2 <- theta[,6]
if (CRcc1) CR.qnt1 <- theta[,3] else CR.qnt1 <- theta[,7]
if (CRcc2) CR.qnt2 <- theta[,4] else CR.qnt2 <- theta[,8]


out <-
list(
TB0.qnt1=theta[,1],
TB0.qnt2=theta[,2],
CR0.qnt1=theta[,3],
CR0.qnt2=theta[,4],

TB1.qnt1=TB.qnt1,
TB1.qnt2=TB.qnt2,
CR1.qnt1=CR.qnt1,
CR1.qnt2=CR.qnt2,

BCG.qnt1=theta[,9],
BCG.qnt2=theta[,10],

p0.TB=theta[,11],
p0.zero=theta[,12],
p1.TB=theta[,13],
p1.zero=theta[,14],
p1.BCG=theta[,15])
return(out)
}

mixture0 <- function(theta)
{
TB0.qnt1 <- theta[1];
TB0.qnt2 <- theta[2];
CR0.qnt1 <- theta[3];
CR0.qnt2 <- theta[4];
p1 <- theta[5];
p0 <- theta[6];
p2 <- 1-p0-p1

if (distTB=="Wb")
 dist1 <- Wb.dsc(ind.brk,TB0.qnt1,TB0.qnt2,c(qnt1TB,qnt2TB) );
if (distTB=="LN")       {
  sigma   <- (log(TB0.qnt2)-log(TB0.qnt1))/(TB0.x2-TB0.x1)
  mu <- log(TB0.qnt1)-sigma*TB0.x1;
  dist1  <- LN.dsc(ind.brk,mu,sigma,0);
}
if (distTB=="N")        {
  sigma  <- (TB0.qnt2-TB0.qnt1)/(TB0.x2-TB0.x1);
  mu <- TB0.qnt1-sigma*TB0.x1;
  dist1 <- N.dsc(ind.brk,mu,sigma);
}
if (distCR=="Wb")
 dist2 <- Wb.dsc(ind.brk,CR0.qnt1,CR0.qnt2,c(qnt1CR,qnt2CR) );
if (distCR=="LN")       {
  sigma   <- (log(CR0.qnt2)-log(CR0.qnt1))/(CR0.x2-CR0.x1)
  mu <- log(CR0.qnt1)-sigma*CR0.x1;
  dist2  <- LN.dsc(ind.brk,mu,sigma,0);
}


p1f1 <- p1*dist1;
p2f2 <- p2*dist2;

mix  <- c(p0,p1f1+p2f2)

mix[is.na(mix)] <- 1e-300;
p1f1[is.na(p1f1)] <- 1e-300;
p2f2[is.na(p2f2)] <- 1e-300;

return(list(mix=mix,p1TB=p1f1,p2CR=p2f2));
}


mixture1 <- function(theta)
{

TB1.qnt1 <- theta[1]
TB1.qnt2 <- theta[2]
CR1.qnt1 <- theta[3]
CR1.qnt2 <- theta[4]
BCG.qnt1 <- theta[5]
BCG.qnt2 <- theta[6]
p1 <- theta[7]
p0 <- theta[8]
p3 <- theta[9]
p2 <- 1-p0-p1-p3

if (distTB=="Wb")
 dist1 <- Wb.dsc(ind.brk,TB1.qnt1,TB1.qnt2,c(qnt1TB,qnt2TB) );
if (distTB=="LN")       {
  sigma   <- (log(TB1.qnt2)-log(TB1.qnt1))/(TB1.x2-TB1.x1)
  mu <- log(TB1.qnt1)-sigma*TB1.x1;
  dist1  <- LN.dsc(ind.brk,mu,sigma,0);
}
if (distTB=="N")        {
  sigma  <- (TB1.qnt2-TB1.qnt1)/(TB1.x2-TB1.x1);
  mu <- TB1.qnt1-sigma*TB1.x1;
  dist1 <- N.dsc(ind.brk,mu,sigma);
}
if (distCR=="Wb")
 dist2 <- Wb.dsc(ind.brk,CR1.qnt1,CR1.qnt2,c(qnt1CR,qnt2CR) );
if (distCR=="LN")       {
  sigma   <- (log(CR1.qnt2)-log(CR1.qnt1))/(CR1.x2-CR1.x1)
  mu <- log(CR1.qnt1)-sigma*CR1.x1;
  dist2  <- LN.dsc(ind.brk,mu,sigma,0);
}

if (distBCG=="Wb")
 dist3 <- Wb.dsc(ind.brk,BCG.qnt1,BCG.qnt2,c(qnt1BCG,qnt2BCG) );

if (distBCG=="LN")       {
  sigma  <- (log(BCG.qnt2)-log(BCG.qnt1))/(BCG.x2-BCG.x1);
  mu <- log(BCG.qnt1)-sigma*BCG.x1;
  dist3 <- LN.dsc(ind.brk,mu,sigma,0);
}

if (distBCG=="N")        {
  sigma  <- (BCG.qnt2-BCG.qnt1)/(BCG.x2-BCG.x1);
  mu <- BCG.qnt1-sigma*BCG.x1;
  dist3 <- N.dsc(ind.brk,mu,sigma);
}

p1f1 <- p1*dist1
p3f3 <- p3*dist3
p2f2 <- p2*dist2

mix  <- c(p0,p1f1+p2f2+p3f3)

mix[is.na(mix)] <- 1e-300;
p1f1[is.na(p1f1)] <- 1e-300;
p2f2[is.na(p2f2)] <- 1e-300;
p3f3[is.na(p3f3)] <- 1e-300;

return(list(mix=mix,p1TB=p1f1,p2CR=p2f2,p3BCG=p3f3));
}



############################################################
sink(file=outfilelog,append=T)
 cat(" ...Create grouping information","\n")
sink()
Sys.sleep(1)
############################################################

if (is.vector(group.names) & !is.list(group.names) )     {

groups.labels <- group.names
groups.numbers <- 1
groups.indices <- matrix(NA,length(group.names),10)

if ( all(TB0.qnt1.diff==0) & all(TB0.qnt1.ratio==1))
groups.indices[,1] <- rep(1,length(group.names))
else
groups.indices[,1] <- seq(1,length(group.names))

if (all(TB0.qnt2.diff==0) & all(TB0.qnt2.ratio==1))
groups.indices[,2] <- rep(1,length(group.names))
else
groups.indices[,2] <- seq(1,length(group.names))

if (all(CR0.qnt1.diff==0) & all(CR0.qnt1.ratio==1))
groups.indices[,3] <- rep(1,length(group.names))
else
groups.indices[,3] <- seq(1,length(group.names))

if (all(CR0.qnt2.diff==0) & all(CR0.qnt2.ratio==1))
groups.indices[,4] <- rep(1,length(group.names))
else
groups.indices[,4] <- seq(1,length(group.names))

if (all(TB1.qnt1.diff==0) & all(TB1.qnt1.ratio==1))
groups.indices[,5] <- rep(1,length(group.names))
else
groups.indices[,5] <- seq(1,length(group.names))

if (all(TB1.qnt2.diff==0) & all(TB1.qnt2.ratio==1))
groups.indices[,6] <- rep(1,length(group.names))
else
groups.indices[,6] <- seq(1,length(group.names))

if (all(CR1.qnt1.diff==0) & all(CR1.qnt1.ratio==1))
groups.indices[,7] <- rep(1,length(group.names))
else
groups.indices[,7] <- seq(1,length(group.names))

if (all(CR1.qnt2.diff==0) & all(CR1.qnt2.ratio==1))
groups.indices[,8] <- rep(1,length(group.names))
else
groups.indices[,8] <- seq(1,length(group.names))

if (all(BCG.qnt1.diff==0) & all(BCG.qnt1.ratio==1))
groups.indices[,9] <- rep(1,length(group.names))
else
groups.indices[,9] <- seq(1,length(group.names))

if (all(BCG.qnt2.diff==0) & all(BCG.qnt2.ratio==1))
groups.indices[,10] <- rep(1,length(group.names))
else
groups.indices[,10] <- seq(1,length(group.names))

}

if (is.list(group.names))       {

if ( length(group.names) != length(group.names[[1]]) + 1 )
stop("Invalid input for group.names")

group.names1 <- group.names[2:length(group.names)]
groups.labels <- MakeGroupNames(group.names1)


groups.numbers <- length(group.names)-1
groups.sizes <- numeric(0)
for (j in 2:(groups.numbers+1) )    {
groups.sizes[j-1] <- length(group.names[[j]])
}

groups.indices <- matrix(NA,length(freq0.column),10)

if (length(TB0.qnt1.diff)==1) TB0.qnt1.diff <- rep(TB0.qnt1.diff,groups.numbers)
if (length(TB0.qnt2.diff)==1) TB0.qnt2.diff <- rep(TB0.qnt2.diff,groups.numbers)
if (length(CR0.qnt1.diff)==1) CR0.qnt1.diff <- rep(CR0.qnt1.diff,groups.numbers)
if (length(CR0.qnt2.diff)==1) CR0.qnt2.diff <- rep(CR0.qnt2.diff,groups.numbers)

if (length(TB0.qnt1.ratio)==1) TB0.qnt1.ratio <- rep(TB0.qnt1.ratio,groups.numbers)
if (length(TB0.qnt2.ratio)==1) TB0.qnt2.ratio <- rep(TB0.qnt2.ratio,groups.numbers)
if (length(CR0.qnt1.ratio)==1) CR0.qnt1.ratio <- rep(CR0.qnt1.ratio,groups.numbers)
if (length(CR0.qnt2.ratio)==1) CR0.qnt2.ratio <- rep(CR0.qnt2.ratio,groups.numbers)

if (length(TB1.qnt1.diff)==1) TB1.qnt1.diff <- rep(TB1.qnt1.diff,groups.numbers)
if (length(TB1.qnt2.diff)==1) TB1.qnt2.diff <- rep(TB1.qnt2.diff,groups.numbers)
if (length(CR1.qnt1.diff)==1) CR1.qnt1.diff <- rep(CR1.qnt1.diff,groups.numbers)
if (length(CR1.qnt2.diff)==1) CR1.qnt2.diff <- rep(CR1.qnt2.diff,groups.numbers)

if (length(TB1.qnt1.ratio)==1) TB1.qnt1.ratio <- rep(TB1.qnt1.ratio,groups.numbers)
if (length(TB1.qnt2.ratio)==1) TB1.qnt2.ratio <- rep(TB1.qnt2.ratio,groups.numbers)
if (length(CR1.qnt1.ratio)==1) CR1.qnt1.ratio <- rep(CR1.qnt1.ratio,groups.numbers)
if (length(CR1.qnt2.ratio)==1) CR1.qnt2.ratio <- rep(CR1.qnt2.ratio,groups.numbers)

if (length(BCG.qnt1.diff)==1) BCG.qnt1.diff <- rep(BCG.qnt1.diff,groups.numbers)
if (length(BCG.qnt2.diff)==1) BCG.qnt2.diff <- rep(BCG.qnt2.diff,groups.numbers)

if (length(BCG.qnt1.ratio)==1) BCG.qnt1.ratio <- rep(BCG.qnt1.ratio,groups.numbers)
if (length(BCG.qnt2.ratio)==1) BCG.qnt2.ratio <- rep(BCG.qnt2.ratio,groups.numbers)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(TB0.qnt1.diff[[j]]==0) & all(TB0.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,1] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(TB0.qnt2.diff[[j]]==0) & all(TB0.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,2] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(CR0.qnt1.diff[[j]]==0) & all(CR0.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,3] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(CR0.qnt2.diff[[j]]==0) & all(CR0.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,4] <- MakeGroupIndices(iii)


iii <- list()
for (j in 1:groups.numbers) {
if ( all(TB1.qnt1.diff[[j]]==0) & all(TB1.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,5] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(TB1.qnt2.diff[[j]]==0) & all(TB1.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,6] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(CR1.qnt1.diff[[j]]==0) & all(CR1.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,7] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(CR1.qnt2.diff[[j]]==0) & all(CR1.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,8] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(BCG.qnt1.diff[[j]]==0) & all(BCG.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,9] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(BCG.qnt2.diff[[j]]==0) & all(BCG.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,10] <- MakeGroupIndices(iii)
}


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Prepare checks for component distributions","\n")
sink()
Sys.sleep(1)
############################################################

TB0.qnt1.diff.PC <- PrepareCheck(group.names,TB0.qnt1.diff,"diff")
TB0.qnt2.diff.PC <- PrepareCheck(group.names,TB0.qnt2.diff,"diff")
CR0.qnt1.diff.PC <- PrepareCheck(group.names,CR0.qnt1.diff,"diff")
CR0.qnt2.diff.PC <- PrepareCheck(group.names,CR0.qnt2.diff,"diff")

TB0.qnt1.ratio.PC <- PrepareCheck(group.names,TB0.qnt1.ratio,"ratio")
TB0.qnt2.ratio.PC <- PrepareCheck(group.names,TB0.qnt2.ratio,"ratio")
CR0.qnt1.ratio.PC <- PrepareCheck(group.names,CR0.qnt1.ratio,"ratio")
CR0.qnt2.ratio.PC <- PrepareCheck(group.names,CR0.qnt2.ratio,"ratio")

TB1.qnt1.diff.PC <- PrepareCheck(group.names,TB1.qnt1.diff,"diff")
TB1.qnt2.diff.PC <- PrepareCheck(group.names,TB1.qnt2.diff,"diff")
CR1.qnt1.diff.PC <- PrepareCheck(group.names,CR1.qnt1.diff,"diff")
CR1.qnt2.diff.PC <- PrepareCheck(group.names,CR1.qnt2.diff,"diff")

TB1.qnt1.ratio.PC <- PrepareCheck(group.names,TB1.qnt1.ratio,"ratio")
TB1.qnt2.ratio.PC <- PrepareCheck(group.names,TB1.qnt2.ratio,"ratio")
CR1.qnt1.ratio.PC <- PrepareCheck(group.names,CR1.qnt1.ratio,"ratio")
CR1.qnt2.ratio.PC <- PrepareCheck(group.names,CR1.qnt2.ratio,"ratio")

BCG.qnt1.diff.PC <- PrepareCheck(group.names,BCG.qnt1.diff,"diff")
BCG.qnt2.diff.PC <- PrepareCheck(group.names,BCG.qnt2.diff,"diff")
BCG.qnt1.ratio.PC <- PrepareCheck(group.names,BCG.qnt1.ratio,"ratio")
BCG.qnt2.ratio.PC <- PrepareCheck(group.names,BCG.qnt2.ratio,"ratio")




############################################################
sink(file=outfilelog,append=T)
 cat(" ...Create initial values for model parameters","\n")
sink()
Sys.sleep(1)
############################################################

if ( !is.na(match("MS.InitialValues",obj))  )
theta00 <- MS.InitialValues

if ( is.na(match("MS.InitialValues",obj)) ) {

p0.zero.init <- freq0zero[1,]/n0.total
p1.zero.init <- freq1zero[1,]/n1.total

if (is.na(match("p0.TB.init",obj)))      {
if (num.cova>1) {
if (ind.num==30)
 p0.TB.init <- apply(freq0zero[11:31,],2,sum)/n0.total
 if (ind.num!=30)
 p0.TB.init <- (1-p0.zero.init)/2
}
if (num.cova==1) {
if (ind.num==30)
 p0.TB.init <- sum(freq0zero[11:31,])/n0.total
if (ind.num!=30)
 p0.TB.init <- (1-p0.zero.init)/2
}
}

if (is.na(match("p1.TB.init",obj)))      {

if (num.cova>1) {
if (ind.num==30)
 p1.TB.init <- apply(freq1zero[11:31,],2,sum)/n1.total/2
 if (ind.num!=30)
 p1.TB.init <- (1-p1.zero.init)/2/2
}

if (num.cova==1) {
if (ind.num==30)
 p1.TB.init <- sum(freq1zero[11:31,])/n1.total/2
if (ind.num!=30)
 p1.TB.init <- (1-p1.zero.init)/2/2
}
}


if (is.na(match("TB0.qnt1.init",obj)))   TB0.qnt1.init <- 18
if (is.na(match("TB0.qnt2.init",obj)))   TB0.qnt2.init <- 30
if (is.na(match("CR0.qnt1.init",obj)))   CR0.qnt1.init <- 2
if (is.na(match("CR0.qnt2.init",obj)))   CR0.qnt2.init <- 5

if (is.na(match("TB1.qnt1.init",obj)))   TB1.qnt1.init <- 18
if (is.na(match("TB1.qnt2.init",obj)))   TB1.qnt2.init <- 30
if (is.na(match("CR1.qnt1.init",obj)))   CR1.qnt1.init <- 2
if (is.na(match("CR1.qnt2.init",obj)))   CR1.qnt2.init <- 5

if (is.na(match("BCG.qnt1.init",obj)))   BCG.qnt1.init <- 8
if (is.na(match("BCG.qnt2.init",obj)))   BCG.qnt2.init <- 15

theta00 <- matrix(NA,num.cova,15)
theta00[,11] <- p0.TB.init
theta00[,12] <- p0.zero.init
theta00[,13] <- p1.TB.init
theta00[,14] <- p1.zero.init
theta00[,15] <- (1-p1.TB.init-p1.zero.init)/2
theta00[,1] <- TB0.qnt1.init;
theta00[,2] <- TB0.qnt2.init;
theta00[,3] <- CR0.qnt1.init;
theta00[,4] <- CR0.qnt2.init;
theta00[,5] <- TB1.qnt1.init
theta00[,6] <- TB1.qnt2.init
theta00[,7] <- CR1.qnt1.init
theta00[,8] <- CR1.qnt2.init
theta00[,9] <- BCG.qnt1.init
theta00[,10] <- BCG.qnt2.init

}

theta00[-groups.indices[,1],1] <- NA
theta00[-groups.indices[,2],2] <- NA
theta00[-groups.indices[,3],3] <- NA
theta00[-groups.indices[,4],4] <- NA

theta00[-groups.indices[,5],5] <- NA
theta00[-groups.indices[,6],6] <- NA
theta00[-groups.indices[,7],7] <- NA
theta00[-groups.indices[,8],8] <- NA
theta00[-groups.indices[,9],9] <- NA
theta00[-groups.indices[,10],10] <- NA

if (TBcc1) theta00[,5] <- theta00[,1]
if (TBcc2) theta00[,6] <- theta00[,2]
if (CRcc1) theta00[,7] <- theta00[,3]
if (CRcc2) theta00[,8] <- theta00[,4]


theta0 <- theta00
dimnames(theta0) <- list(NULL,NULL)

dimnames(theta00) <- list(groups.labels,c(
"TB0qnt1","TB0qnt2",
"CR0qnt1","CR0qnt2",
"TB1qnt1","TB1qnt2",
"CR1qnt1","CR1qnt2",
"BCGqnt1","BCGqnt2",
"p0TB","p0zero",
"p1TB","p1zero","p1BCG"))


cat("Initial Values","\n")
print(theta00,digits=3)
llinits <- loglik(theta00,0)
cat("\n")
cat("Log-likelihood at initial values:",llinits,"\n")

if (llinits <= -1e300)
stop("Numerical problems: try different starting values for parameters")

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Check initial model parameters","\n")
sink()
Sys.sleep(1)
############################################################

#       check initial parameter matrix
th0 <- theta0
th0[,1] <- theta0[groups.indices[,1],1]
th0[,2] <- theta0[groups.indices[,2],2]
th0[,3] <- theta0[groups.indices[,3],3]
th0[,4] <- theta0[groups.indices[,4],4]
th0[,5] <- theta0[groups.indices[,5],5]
th0[,6] <- theta0[groups.indices[,6],6]
th0[,7] <- theta0[groups.indices[,7],7]
th0[,8] <- theta0[groups.indices[,8],8]
th0[,9] <- theta0[groups.indices[,9],9]
th0[,10] <- theta0[groups.indices[,10],10]

p1      <- th0[,11];
p0      <- th0[,12];
TB0.qnt1  <- th0[,1];
TB0.qnt2  <- th0[,2];
CR0.qnt1  <- th0[,3];
CR0.qnt2  <- th0[,4];
TB1.qnt1  <- th0[,5];
TB1.qnt2  <- th0[,6];
CR1.qnt1  <- th0[,7];
CR1.qnt2  <- th0[,8];
BCG.qnt1  <- th0[,9];
BCG.qnt2  <- th0[,10];

cond1a <- Checks(TB0.qnt1,TB0.qnt1.diff.PC)
cond2a <- Checks(TB0.qnt2,TB0.qnt2.diff.PC)
cond3a <- Checks(CR0.qnt1,CR0.qnt1.diff.PC)
cond4a <- Checks(CR0.qnt2,CR0.qnt2.diff.PC)

cond5a <- Checks(TB1.qnt1,TB1.qnt1.diff.PC)
cond6a <- Checks(TB1.qnt2,TB1.qnt2.diff.PC)
cond7a <- Checks(CR1.qnt1,CR1.qnt1.diff.PC)
cond8a <- Checks(CR1.qnt2,CR1.qnt2.diff.PC)

cond9a <- Checks(BCG.qnt1,BCG.qnt1.diff.PC)
cond10a <- Checks(BCG.qnt2,BCG.qnt2.diff.PC)

cond1b <- Checks(TB0.qnt1,TB0.qnt1.ratio.PC)
cond2b <- Checks(TB0.qnt2,TB0.qnt2.ratio.PC)
cond3b <- Checks(CR0.qnt1,CR0.qnt1.ratio.PC)
cond4b <- Checks(CR0.qnt2,CR0.qnt2.ratio.PC)

cond5b <- Checks(TB1.qnt1,TB1.qnt1.ratio.PC)
cond6b <- Checks(TB1.qnt2,TB1.qnt2.ratio.PC)
cond7b <- Checks(CR1.qnt1,CR1.qnt1.ratio.PC)
cond8b <- Checks(CR1.qnt2,CR1.qnt2.ratio.PC)

cond9b <- Checks(BCG.qnt1,BCG.qnt1.ratio.PC)
cond10b <- Checks(BCG.qnt2,BCG.qnt2.ratio.PC)

cond <- c(
cond1a,cond2a,cond3a,cond4a,cond5a,cond6a,cond7a,cond8a,cond9a,cond10a,
cond1b,cond2b,cond3b,cond4b,cond5b,cond6b,cond7b,cond8b,cond9b,cond10b
)

if (!all(cond))
{
if (!cond1a) stop("Initial values for TB0.qnt1 not consistent with TB0.qnt1.diff")
if (!cond2a) stop("Initial values for TB0.qnt2 not consistent with TB0.qnt2.diff")
if (!cond3a) stop("Initial values for CR0.qnt1 not consistent with CR0.qnt1.diff")
if (!cond4a) stop("Initial values for CR0.qnt2 not consistent with CR0.qnt2.diff")
if (!cond5a) stop("Initial values for TB1.qnt1 not consistent with TB1.qnt1.diff")
if (!cond6a) stop("Initial values for TB1.qnt2 not consistent with TB1.qnt2.diff")
if (!cond7a) stop("Initial values for CR1.qnt1 not consistent with CR1.qnt1.diff")
if (!cond8a) stop("Initial values for CR1.qnt2 not consistent with CR1.qnt2.diff")
if (!cond9a) stop("Initial values for BCG.qnt1 not consistent with BCG.qnt1.diff")
if (!cond10a) stop("Initial values for BCGqnt2 not consistent with BCG.qnt2.diff")

if (!cond1b) stop("Initial values for TB0.qnt1 not consistent with TB0.qnt1.ratio")
if (!cond2b) stop("Initial values for TB0.qnt2 not consistent with TB0.qnt2.ratio")
if (!cond3b) stop("Initial values for CR0.qnt1 not consistent with CR0.qnt1.ratio")
if (!cond4b) stop("Initial values for CR0.qnt2 not consistent with CR0.qnt2.ratio")
if (!cond5b) stop("Initial values for TB1.qnt1 not consistent with TB1.qnt1.ratio")
if (!cond6b) stop("Initial values for TB1.qnt2 not consistent with TB1.qnt2.ratio")
if (!cond7b) stop("Initial values for CR1.qnt1 not consistent with CR1.qnt1.ratio")
if (!cond8b) stop("Initial values for CR1.qnt2 not consistent with CR1.qnt2.ratio")
if (!cond9b) stop("Initial values for BCG.qnt1 not consistent with BCG.qnt1.ratio")
if (!cond10b) stop("Initial values for BCGqnt2 not consistent with BCG.qnt2.ratio")
}

if ( !all(TB0.qnt1<TB0.qnt2) ) stop("Initial values TB0.qnt1<TB0.qnt2 violation")
if ( !all(TB1.qnt1<TB1.qnt2) ) stop("Initial values TB1.qnt1<TB1.qnt2 violation")
if ( !all(CR0.qnt1<CR0.qnt2) ) stop("Initial values CR0.qnt1<CR0.qnt2 violation")
if ( !all(CR1.qnt1<CR1.qnt2) ) stop("Initial values CR1.qnt1<CR1.qnt2 violation")
if ( !all(BCG.qnt1<BCG.qnt2) ) stop("Initial values BCG.qnt1<BCG.qnt2 violation")

if ( !all(CR0.qnt1<BCG.qnt1) ) stop("Initial values CR0.qnt1<BCG.qnt1 violation")
if ( !all(CR1.qnt1<BCG.qnt1) ) stop("Initial values CR1.qnt1<BCG.qnt1 violation")
if ( !all(CR0.qnt2<BCG.qnt2) ) stop("Initial values CR0.qnt2<BCG.qnt2 violation")
if ( !all(CR1.qnt2<BCG.qnt2) ) stop("Initial values CR1.qnt2<BCG.qnt2 violation")

if ( !all(BCG.qnt1<TB0.qnt1) ) stop("Initial values BCG.qnt1<TB0.qnt1 violation")
if ( !all(BCG.qnt1<TB1.qnt1) ) stop("Initial values BCG.qnt1<TB1.qnt1 violation")
if ( !all(BCG.qnt2<TB0.qnt2) ) stop("Initial values BCG.qnt2<TB0.qnt2 violation")
if ( !all(BCG.qnt2<TB1.qnt2) ) stop("Initial values BCG.qnt2<TB1.qnt2 violation")

if ( any(th0[,11]+th0[,12]>=1) ) stop("p0+p1>1 (non-BCG) violation")
if ( any(th0[,13]+th0[,14]+th0[,15]>=1) ) stop("p0+p1+p2>1 (BCG) violation")

cond22 <- all(CR1.qnt1<TB1.qnt1)
cond23 <- all(CR1.qnt2<TB1.qnt2)

cond30 <- all(BCG.qnt1<BCG.qnt2)
cond31 <- all(BCG.qnt1<TB1.qnt1)
cond32 <- all(BCG.qnt2<TB1.qnt2)
cond33 <- all(BCG.qnt1>CR1.qnt1)
cond34 <- all(BCG.qnt2>CR1.qnt2)

cond40 <- all(th0[,13]+th0[,14]+th0[,15]<1)


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Boundaries for parameters","\n")
sink()
Sys.sleep(1)
############################################################

MS.low <- NA*theta0
MS.high <- NA*theta0

MS.low[,1] <- TB0.qnt1.int[1]
MS.low[,2] <- TB0.qnt2.int[1]
MS.low[,3] <- CR0.qnt1.int[1]
MS.low[,4] <- CR0.qnt2.int[1]
MS.low[,5] <- TB1.qnt1.int[1]
MS.low[,6] <- TB1.qnt2.int[1]
MS.low[,7] <- CR1.qnt1.int[1]
MS.low[,8] <- CR1.qnt2.int[1]
MS.low[,9] <- BCG.qnt1.int[1]
MS.low[,10] <- BCG.qnt2.int[1]

MS.low[,11] <- p0.TB.int[1]
MS.low[,12] <- p0.zero.int[1]
MS.low[,13] <- p1.TB.int[1]
MS.low[,14] <- p1.zero.int[1]
MS.low[,15] <- p1.BCG.int[1]


MS.high[,1] <- TB0.qnt1.int[2]
MS.high[,2] <- TB0.qnt2.int[2]
MS.high[,3] <- CR0.qnt1.int[2]
MS.high[,4] <- CR0.qnt2.int[2]
MS.high[,5] <- TB1.qnt1.int[2]
MS.high[,6] <- TB1.qnt2.int[2]
MS.high[,7] <- CR1.qnt1.int[2]
MS.high[,8] <- CR1.qnt2.int[2]
MS.high[,9] <- BCG.qnt1.int[2]
MS.high[,10] <- BCG.qnt2.int[2]

MS.high[,11] <- p0.TB.int[2]
MS.high[,12] <- p0.zero.int[2]
MS.high[,13] <- p1.TB.int[2]
MS.high[,14] <- p1.zero.int[2]
MS.high[,15] <- p1.BCG.int[2]



if (is.na(match("MS.scale",obj)))
MS.scale <- matrix( c(0.1,0.1, 0.05,0.05, 0.1,0.1, 0.05,0.05, 0.05,0.05, 0.01, 0.01, 0.01, 0.01, 0.01 ), nrow(theta0),ncol(theta0), byrow=T)

if (is.na(match("MS.block",obj)))
MS.block <- matrix(c(1,2,3,4, 1,2,3,4,5,6,7,8,7,8,9),nrow(theta0),ncol(theta0),byrow=T)


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Create labels","\n")
sink()
Sys.sleep(1)
############################################################


para0.names <- c(
paste("TB0(",100*qnt1TB,"%) ",groups.labels,sep=""),
paste("TB0(",100*qnt2TB,"%) ",groups.labels,sep=""),
paste("CR0(",100*qnt1CR,"%) ",groups.labels,sep=""),
paste("CR0(",100*qnt2CR,"%) ",groups.labels,sep=""),
paste("p0.TB",groups.labels),
paste("p0.zero",groups.labels),
);

para1.names <- c(
paste("TB1(",100*qnt1TB,"%) ",groups.labels,sep=""),
paste("TB1(",100*qnt2TB,"%) ",groups.labels,sep=""),
paste("CR1(",100*qnt1CR,"%) ",groups.labels,sep=""),
paste("CR1(",100*qnt2CR,"%) ",groups.labels,sep=""),
paste("BCG(",100*qnt1CR,"%) ",groups.labels,sep=""),
paste("BCG(",100*qnt2CR,"%) ",groups.labels,sep=""),
paste("p1.TB",groups.labels),
paste("p1.zero",groups.labels),
);

out.str <- c("mean",      "st.dev",    "2.5%",      "50%",       "97.5%" )

dimnames(freq0all) <- list(ind.namesntot,groups.labels)
dimnames(freq1all) <- list(ind.namesntot,groups.labels)


if (MS.run)     {

############################################################
sink(file=outfilelog,append=T)
 cat("MS.run: Start Metropolis Sampler","\n")
sink()
Sys.sleep(1)
############################################################

cat("\n",paste("Program",Version),"\n");
cat("\n","Metropolis Sampling (MS) algorithm ...","\n")

fit <- Metropolis(data=0,loglik="loglik",logprior="prior0",gibbs="gibbs0",trans="trans0",
theta0=theta0,MS.adj=MS.adj,MS.scale=MS.scale,MS.block=MS.block,
MS.low=MS.low,MS.high=MS.high,MS.size=MS.size,MS.names=MS.names,MS.screen=MS.screen,MS.screen.dig=MS.screen.dig)


}

if (MS.results)         {

############################################################
sink(file=outfilelog,append=T)
 cat("MS.results: Processing results","\n")
sink()
Sys.sleep(1)
############################################################


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Loading sampled values","\n")
sink()
Sys.sleep(1)
############################################################

TB0.qnt1 <- fit$Sample[[1]]
TB0.qnt2 <- fit$Sample[[2]]
CR0.qnt1 <- fit$Sample[[3]]
CR0.qnt2 <- fit$Sample[[4]]
TB1.qnt1 <- fit$Sample[[5]]
TB1.qnt2 <- fit$Sample[[6]]
CR1.qnt1 <- fit$Sample[[7]]
CR1.qnt2 <- fit$Sample[[8]]
BCG.qnt1 <- fit$Sample[[9]]
BCG.qnt2 <- fit$Sample[[10]]

p0.TB <- fit$Sample[[11]]
p0.zero <- fit$Sample[[12]]
p1.TB <- fit$Sample[[13]]
p1.zero <- fit$Sample[[14]]

sink(file=outfile1,append=F);

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Basic input for analysis","\n")
sink()
Sys.sleep(1)
############################################################


cat(date(),"\n","\n");
cat(paste("Program",Version),"\n");
cat("Mixture Analysis of induration data: TB infections (TB), crossreactions (CR), BCG reactions (BCG)","\n","\n");

cat("File specifications","\n")
cat(" Non-BCG data: ",infile0,"\n")
cat(" BCG data:     ",infile1,"\n")
cat(" Output files: ",outfile,"\n","\n")

if (!is.na(match("RndSeed",obj)))
cat("Random seed at start:", RndSeed,"\n","\n")

if (is.list(group.names))
cat("Data grouped by:",group.names[[1]],"\n","\n")


if (ind.num != 30)      {
cat("Grouped induration data with intervals defined by cut-offs:","\n")
cat(ind.brk)
cat("\n","\n")
}

cat("non-BCG frequencies","\n");
print(freq0all);

cat("\n")
cat("BCG frequencies","\n");
print(freq1all);

cat("\n");
cat("Metropolis Sampler:","\n");
cat(" burn-in:           ",MS.size[1],"\n");
cat(" sample size:       ",MS.size[2],"\n");
cat(" thinning:          ",MS.size[3],"\n");
cat(" no. of iterations: ",MS.size[1]+MS.size[2]*MS.size[3],"\n","\n");

cat("Distribution of TB infections:  ", distTB,"\n");
cat("  parameters defined by two quantiles:  ", qnt1TB,qnt2TB,"\n","\n");

cat("Distribution of crossreactions: ", distCR,"\n");
cat("  parameters defined by two quantiles:  ", qnt1CR,qnt2CR,"\n","\n");

cat("Distribution of BCG reactions: ", distBCG,"\n");
cat("  parameters defined by two quantiles:  ", qnt1BCG,qnt2BCG,"\n","\n");


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Parameter constraints","\n")
sink()
Sys.sleep(1)
############################################################


if (TB0.qnt1.diff.PC[[1]])
{
cat("\n")
cat("Differences for TB distribution (1st quantile):","\n")
if (length(TB0.qnt1.diff)==1)            cat(TB0.qnt1.diff,"\n")
if (length(TB0.qnt1.diff)>1)
{
for (j in 1:length(TB0.qnt1.diff))       cat(group.names[[1]][j],TB0.qnt1.diff[[j]],"\n")
}
}

if (TB0.qnt1.ratio.PC[[1]])
{
cat("\n")
cat("Ratios for TB distribution (1st quantile):","\n")
if (length(TB0.qnt1.ratio)==1)            cat(TB0.qnt1.ratio,"\n")
if (length(TB0.qnt1.ratio)>1)
{
for (j in 1:length(TB0.qnt1.ratio))       cat(group.names[[1]][j],TB0.qnt1.ratio[[j]],"\n")
}
}


if (TB0.qnt2.diff.PC[[1]])
{
cat("\n")
cat("Differences for TB distribution (2nd quantile):","\n")
if (length(TB0.qnt2.diff)==1)            cat(TB0.qnt2.diff,"\n")
if (length(TB0.qnt2.diff)>1)
{
for (j in 1:length(TB0.qnt2.diff))       cat(group.names[[1]][j],TB0.qnt2.diff[[j]],"\n")
}
}

if (TB0.qnt2.ratio.PC[[1]])
{
cat("\n")
cat("Ratios for TB distribution (2nd quantile):","\n")
if (length(TB0.qnt2.ratio)==1)            cat(TB0.qnt2.ratio,"\n")
if (length(TB0.qnt2.ratio)>1)
{
for (j in 1:length(TB0.qnt2.ratio))       cat(group.names[[1]][j],TB0.qnt2.ratio[[j]],"\n")
}
}




if (CR0.qnt1.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (1st quantile)","\n")
if (length(CR0.qnt1.diff)==1)            cat(CR0.qnt1.diff,"\n")
if (length(CR0.qnt1.diff)>1)
{
for (j in 1:length(CR0.qnt1.diff))       cat(group.names[[1]][j],CR0.qnt1.diff[[j]],"\n")
}
}

if (CR0.qnt1.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (1st quantile)","\n")
if (length(CR0.qnt1.ratio)==1)            cat(CR0.qnt1.ratio,"\n")
if (length(CR0.qnt1.ratio)>1)
{
for (j in 1:length(CR0.qnt1.ratio))       cat(group.names[[1]][j],CR0.qnt1.ratio[[j]],"\n")
}
}


if (CR0.qnt2.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (2nd quantile)","\n")
if (length(CR0.qnt2.diff)==1)            cat(CR0.qnt2.diff,"\n")
if (length(CR0.qnt2.diff)>1)
{
for (j in 1:length(CR0.qnt2.diff))       cat(group.names[[1]][j],CR0.qnt2.diff[[j]],"\n")
}
}

if (CR0.qnt2.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (2nd quantile)","\n")
if (length(CR0.qnt2.ratio)==1)            cat(CR0.qnt2.ratio,"\n")
if (length(CR0.qnt2.ratio)>1)
{
for (j in 1:length(CR0.qnt2.ratio))       cat(group.names[[1]][j],CR0.qnt2.ratio[[j]],"\n")
}
}


if (TB1.qnt1.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for TB distribution (1st quantile)","\n")
if (length(TB1.qnt1.diff)==1)            cat(TB1.qnt1.diff,"\n")
if (length(TB1.qnt1.diff)>1)
{
for (j in 1:length(TB1.qnt1.diff))       cat(group.names[[1]][j],TB1.qnt1.diff[[j]],"\n")
}
}

if (TB1.qnt1.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for TB distribution (1st quantile)","\n")
if (length(TB1.qnt1.ratio)==1)            cat(TB1.qnt1.ratio,"\n")
if (length(TB1.qnt1.ratio)>1)
{
for (j in 1:length(TB1.qnt1.ratio))       cat(group.names[[1]][j],TB1.qnt1.ratio[[j]],"\n")
}
}


if (TB1.qnt2.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for TB distribution (2nd quantile)","\n")
if (length(TB1.qnt2.diff)==1)            cat(TB1.qnt2.diff,"\n")
if (length(TB1.qnt2.diff)>1)
{
for (j in 1:length(TB1.qnt2.diff))       cat(group.names[[1]][j],TB1.qnt2.diff[[j]],"\n")
}
}

if (TB1.qnt2.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for TB distribution (2nd quantile)","\n")
if (length(TB1.qnt2.ratio)==1)            cat(TB1.qnt2.ratio,"\n")
if (length(TB1.qnt2.ratio)>1)
{
for (j in 1:length(TB1.qnt2.ratio))       cat(group.names[[1]][j],TB1.qnt2.ratio[[j]],"\n")
}
}


if (CR1.qnt1.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (1st quantile)","\n")
if (length(CR1.qnt1.diff)==1)            cat(CR1.qnt1.diff,"\n")
if (length(CR1.qnt1.diff)>1)
{
for (j in 1:length(CR1.qnt1.diff))       cat(group.names[[1]][j],CR1.qnt1.diff[[j]],"\n")
}
}

if (CR1.qnt1.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (1st quantile)","\n")
if (length(CR1.qnt1.ratio)==1)            cat(CR1.qnt1.ratio,"\n")
if (length(CR1.qnt1.ratio)>1)
{
for (j in 1:length(CR1.qnt1.ratio))       cat(group.names[[1]][j],CR1.qnt1.ratio[[j]],"\n")
}
}


if (CR1.qnt2.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (2nd quantile)","\n")
if (length(CR1.qnt2.diff)==1)            cat(CR1.qnt2.diff,"\n")
if (length(CR1.qnt2.diff)>1)
{
for (j in 1:length(CR1.qnt2.diff))       cat(group.names[[1]][j],CR1.qnt2.diff[[j]],"\n")
}
}


if (CR1.qnt2.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (2nd quantile)","\n")
if (length(CR1.qnt2.ratio)==1)            cat(CR1.qnt2.ratio,"\n")
if (length(CR1.qnt2.ratio)>1)
{
for (j in 1:length(CR1.qnt2.ratio))       cat(group.names[[1]][j],CR1.qnt2.ratio[[j]],"\n")
}
}



if (!TBcc1a)
{
cat("\n")
cat("Parameter constraints: Differences for TB distribution (1st quantile), BCG vs. non-BCG","\n")
           cat(TB01.qnt1.diff,"\n")
}

if (!TBcc1b)
{
cat("\n")
cat("Parameter constraints: Ratios for TB distribution (1st quantile), BCG vs. non-BCG","\n")
           cat(TB01.qnt1.ratio,"\n")
}

if (!TBcc2a)
{
cat("\n")
cat("Parameter constraints: Differences for TB distribution (2nd quantile), BCG vs. non-BCG","\n")
           cat(TB01.qnt2.diff,"\n")
}

if (!TBcc2b)
{
cat("\n")
cat("Parameter constraints: Ratios for TB distribution (2nd quantile), BCG vs. non-BCG","\n")
           cat(TB01.qnt2.ratio,"\n")
}

if (!CRcc1a)
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (1st quantile), BCG vs. non-BCG","\n")
           cat(CR01.qnt1.diff,"\n")
}

if (!CRcc1b)
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (1st quantile), BCG vs. non-BCG","\n")
           cat(CR01.qnt1.ratio,"\n")
}

if (!CRcc2a)
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (2nd quantile), BCG vs. non-BCG","\n")
           cat(CR01.qnt2.diff,"\n")
}

if (!CRcc2b)
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (2nd quantile), BCG vs. non-BCG","\n")
           cat(CR01.qnt2.ratio,"\n")
}



cat("\n")
cat("Maximum log-likelihood (from simulation): ", -fit$Deviance$Dhat.mpe/2,"\n","\n");

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Posterior summaries","\n")
sink()
Sys.sleep(1)
############################################################


cat("Parameter estimates: non-BCG","\n")

cat("\n","Prevalence of TB infections (p0.TB)","\n")
p0.TB.summary <- MSoutput(p0.TB,c(0.025,0.50,0.975),groups.labels)
print(p0.TB.summary,digits=3)

cat("\n","Prevalence of no reaction (p0.zero)","\n")
p0.zero.summary <- MSoutput(p0.zero,c(0.025,0.50,0.975),groups.labels)
print(p0.zero.summary,digits=3)

cat("\n","TB distribution (1st quantile, TB0.qnt1)","\n")
TB0.qnt1.summary <- MSoutput(TB0.qnt1,c(0.025,0.50,0.975),groups.labels)
print(TB0.qnt1.summary,digits=3)

cat("\n","TB distribution (2nd quantile, TB0.qnt2)","\n")
TB0.qnt2.summary <- MSoutput(TB0.qnt2,c(0.025,0.50,0.975),groups.labels)
print(TB0.qnt2.summary,digits=3)

cat("\n","CR distribution (1st quantile, CR0.qnt1)","\n")
CR0.qnt1.summary <- MSoutput(CR0.qnt1,c(0.025,0.50,0.975),groups.labels)
print(CR0.qnt1.summary,digits=3)

cat("\n","CR distribution (2nd quantile, CR0.qnt2)","\n")
CR0.qnt2.summary <- MSoutput(CR0.qnt2,c(0.025,0.50,0.975),groups.labels)
print(CR0.qnt2.summary,digits=3)

cat("\n")
cat("Parameter estimates: BCG","\n")

cat("\n","Prevalence of TB infections (p1.TB)","\n")
p1.TB.summary <- MSoutput(p1.TB,c(0.025,0.50,0.975),groups.labels)
print(p1.TB.summary,digits=3)

cat("\n","Prevalence of no reaction (p1.zero)","\n")
p1.zero.summary <- MSoutput(p1.zero,c(0.025,0.50,0.975),groups.labels)
print(p1.zero.summary,digits=3)

cat("\n","TB distribution (1st quantile, TB1.qnt1)","\n")
TB1.qnt1.summary <- MSoutput(TB1.qnt1,c(0.025,0.50,0.975),groups.labels)
print( TB1.qnt1.summary,digits=3 )

cat("\n","TB distribution (2nd quantile, TB1.qnt2)","\n")
TB1.qnt2.summary <- MSoutput(TB1.qnt2,c(0.025,0.50,0.975),groups.labels)
print( TB1.qnt2.summary,digits=3)

cat("\n","CR distribution (1st quantile, CR1.qnt1)","\n")
CR1.qnt1.summary <- MSoutput(CR1.qnt1,c(0.025,0.50,0.975),groups.labels)
print( CR1.qnt1.summary, digits=3 )

cat("\n","CR distribution (2nd quantile, CR1.qnt2)","\n")
CR1.qnt2.summary <- MSoutput(CR1.qnt2,c(0.025,0.50,0.975),groups.labels)
print( CR1.qnt2.summary, digits=3 )

cat("\n","BCG distribution (1st quantile, BCG.qnt1)","\n")
BCG.qnt1.summary <- MSoutput(BCG.qnt1,c(0.025,0.50,0.975),groups.labels)
print( BCG.qnt1.summary, digits=3 )

cat("\n","BCG distribution (2nd quantile, BCG.qnt2)","\n")
BCG.qnt2.summary <- MSoutput(BCG.qnt2,c(0.025,0.50,0.975),groups.labels)
print( BCG.qnt2.summary, digits=3 )

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Maximum posterior estimates ","\n")
sink()
Sys.sleep(1)
############################################################


mpe00 <- fit$Estimates$mpe

mpe0 <- theta0*NA
mpe0[,11] <- mpe00[[11]]
mpe0[,12] <- mpe00[[12]]
mpe0[,13] <- mpe00[[13]]
mpe0[,14] <- mpe00[[14]]
mpe0[,15] <- mpe00[[15]]

mpe0[unique(groups.indices[,1]),1] <- mpe00[[1]]
mpe0[unique(groups.indices[,2]),2] <- mpe00[[2]]
mpe0[unique(groups.indices[,3]),3] <- mpe00[[3]]
mpe0[unique(groups.indices[,4]),4] <- mpe00[[4]]
mpe0[unique(groups.indices[,5]),5] <- mpe00[[5]]
mpe0[unique(groups.indices[,6]),6] <- mpe00[[6]]
mpe0[unique(groups.indices[,7]),7] <- mpe00[[7]]
mpe0[unique(groups.indices[,8]),8] <- mpe00[[8]]
mpe0[unique(groups.indices[,9]),9] <- mpe00[[9]]
mpe0[unique(groups.indices[,10]),10] <- mpe00[[10]]


mpe0[,1] <- mpe0[groups.indices[,1],1]
mpe0[,2] <- mpe0[groups.indices[,2],2]
mpe0[,3] <- mpe0[groups.indices[,3],3]
mpe0[,4] <- mpe0[groups.indices[,4],4]
mpe0[,5] <- mpe0[groups.indices[,5],5]
mpe0[,6] <- mpe0[groups.indices[,6],6]
mpe0[,7] <- mpe0[groups.indices[,7],7]
mpe0[,8] <- mpe0[groups.indices[,8],8]
mpe0[,9] <- mpe0[groups.indices[,9],9]
mpe0[,10] <- mpe0[groups.indices[,10],10]

if (TBcc1) mpe0[,5] <- mpe0[,1]
if (TBcc2) mpe0[,6] <- mpe0[,2]
if (CRcc1) mpe0[,7] <- mpe0[,3]
if (CRcc2) mpe0[,8] <- mpe0[,4]

#       summary of posterior parameter estimates
cat("\n");
cat("Maximum posterior estimates","\n");
dimnames(mpe0) <- list(groups.labels,MS.names)
print(mpe0,digits=3);


LastValues <- fit$Output$theta.last
LastValues[,1] <- LastValues[groups.indices[,1],1]
LastValues[,2] <- LastValues[groups.indices[,2],2]
LastValues[,3] <- LastValues[groups.indices[,3],3]
LastValues[,4] <- LastValues[groups.indices[,4],4]
LastValues[,5] <- LastValues[groups.indices[,5],5]
LastValues[,6] <- LastValues[groups.indices[,6],6]
LastValues[,7] <- LastValues[groups.indices[,7],7]
LastValues[,8] <- LastValues[groups.indices[,8],8]
LastValues[,9] <- LastValues[groups.indices[,9],9]
LastValues[,10] <- LastValues[groups.indices[,10],10]

if (TBcc1) LastValues[,5] <- LastValues[,1]
if (TBcc2) LastValues[,6] <- LastValues[,2]
if (CRcc1) LastValues[,7] <- LastValues[,3]
if (CRcc2) LastValues[,8] <- LastValues[,4]



############################################################
sink(file=outfilelog,append=T)
 cat(" ...Total prevalences","\n")
sink()
Sys.sleep(1)
############################################################

p0.TB.total <- matrix(0,nrow(p0.TB),1);
p0.zero.total <- p0.TB.total;

for (i in 1:nrow(p0.TB) )   {
for (j in 1:num.cova)       {
p0.TB.total[i] <- p0.TB.total[i]+n0.total[j]*p0.TB[i,j]
p0.zero.total[i]  <- p0.zero.total[i]+n0.total[j]*p0.zero[i,j]
}
}

p0.TB.total <- p0.TB.total/sum(n0.total);
p0.zero.total <- p0.zero.total/sum(n0.total);

p0.TB.total.sum <- MSoutput(p0.TB.total,c(0.025,0.50,0.975),"p0.TB");
p0.zero.total.sum <- MSoutput(p0.zero.total,c(0.025,0.50,0.975),"p0.zero");

cat("\n","Total prevalence of TB infections (non-BCG)","\n");
print(p0.TB.total.sum,digits=3);
cat("\n","Total prevalence of zero reaction (non-BCG)","\n");
print(p0.zero.total.sum,digits=3);

p1.TB.total <- matrix(0,nrow(p1.TB),1);
p1.zero.total <- p1.TB.total;

for (i in 1:nrow(p1.TB) )   {
for (j in 1:num.cova)       {
p1.TB.total[i] <- p1.TB.total[i]+n1.total[j]*p1.TB[i,j]
p1.zero.total[i]  <- p1.zero.total[i]+n1.total[j]*p1.zero[i,j]
}
}

p1.TB.total <- p1.TB.total/sum(n1.total);
p1.zero.total <- p1.zero.total/sum(n1.total);

p1.TB.total.sum <- MSoutput(p1.TB.total,c(0.025,0.50,0.975),"p1.TB");
p1.zero.total.sum <- MSoutput(p1.zero.total,c(0.025,0.50,0.975),"p1.zero");

cat("\n","Total prevalence of TB infections (BCG)","\n");
print(p1.TB.total.sum,digits=3);
cat("\n","Total prevalence of zero reaction (BCG)","\n");
print(p1.zero.total.sum,digits=3);

cat("\n","\n")

cat("Initial Values","\n")
print(theta00,digits=3)
llinits <- loglik(theta00,0)
cat("\n")
cat("Log-likelihood at initial values:",llinits,"\n")

sink()
Sys.sleep(1)
############################################################
sink(file=outfilelog,append=T)
 cat(" ...Fitted frequencies","\n")
sink()
Sys.sleep(1)
############################################################

sink(file=outfile2,append=F);

############################################################
sink(file=outfilelog,append=T)
 cat(" ...non-BCG","\n")
sink()
Sys.sleep(1)
############################################################


cat("\n")
cat("          non-BCG","\n")

mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;

for (j in 1:num.cova)   {
cat("\n",paste(groups.labels[j],"(non-BCG)"),"\n");
ffreq0 <- freq0[,j];
theta <- numeric(0);
theta <- mpe0[j,c(1,2,3,4,11,12)]
dist <- mixture0(theta);
mix <- dist$mix;
n <- sum(freq0zero[,j]);
distout <- cbind(freq0zero[,j],n*dist$mix,c(0,n*dist$p1TB),c(0,n*dist$p2CR));
dimnames(distout) <- list( ind.names0,c("obs","mixture","p1*TB", "p2*CR"));
print(round(distout,3));
mixobs[,j] <- mix[2:(ind.num+1)]/sum(mix[2:(ind.num+1)])*sum(freq0[,j]);
}

cat("\n")
cat("          BCG","\n")

############################################################
sink(file=outfilelog,append=T)
 cat(" ...BCG","\n")
sink()
Sys.sleep(1)
############################################################


mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;
p3f3obs <- mixobs;

for (j in 1:num.cova)   {
cat("\n",paste(groups.labels[j],"(BCG)"),"\n");
ffreq1 <- freq1[,j];
theta <- numeric(0);
theta <- mpe0[j,c(5,6,7,8,9,10,13,14,15)]
dist <- mixture1(theta);
mix <- dist$mix;
n <- sum(freq1zero[,j]);
distout <- cbind(freq1zero[,j],n*dist$mix,c(0,n*dist$p1TB),c(0,n*dist$p2CR),c(0,n*dist$p3BCG));
dimnames(distout) <- list(ind.names0,c("obs","mixture","p1*TB", "p2*CR", "p3*BCG"));
print(round(distout,3));
mixobs[,j] <- mix[2:(ind.num+1)]/sum(mix[2:(ind.num+1)])*sum(freq1[,j]);
}

sink();
Sys.sleep(1)
} # end of analysis section


if (MS.check)           {
#       posterior predictive checks

############################################################
sink(file=outfilelog,append=T)
 cat("MS.check: Predictive model checks","\n")
sink()
Sys.sleep(1)
############################################################


TB0.qnt1 <- fit$Sample[[1]]
TB0.qnt2 <- fit$Sample[[2]]
CR0.qnt1 <- fit$Sample[[3]]
CR0.qnt2 <- fit$Sample[[4]]
TB1.qnt1 <- fit$Sample[[5]]
TB1.qnt2 <- fit$Sample[[6]]
CR1.qnt1 <- fit$Sample[[7]]
CR1.qnt2 <- fit$Sample[[8]]
BCG.qnt1 <- fit$Sample[[9]]
BCG.qnt2 <- fit$Sample[[10]]


p0.TB <- fit$Sample[[11]]
p0.zero <- fit$Sample[[12]]
p1.TB <- fit$Sample[[13]]
p1.zero <- fit$Sample[[14]]
p1.BCG <- fit$Sample[[15]]

sink(file=outfile3,append=F);

pred0.table <- matrix(0,ind.num,num.cova)
pred1.table <- matrix(0,ind.num,num.cova)

cat("\n","\n")
cat(
"Model checks: observed vs. predicted frequencies (difference)","\n");
cat(
"-------------------------------------------------------------","\n","\n")


############################################################
sink(file=outfilelog,append=T)
 cat(" ...non-BCG","\n")
sink()
Sys.sleep(1)
############################################################

for (j in 1:num.cova)   {
sink();
Sys.sleep(1)
cat("\n","posterior predictive model check: ",paste(groups.labels[j],"(non-BCG)"));

sink(file=outfile3,append=T);

cat(paste(groups.labels[j],"(non BGG)"),"\n");
ffreq0 <- freq0[,j];
pred0mat <- matrix(0,nrow(p0.TB),length(ind));

if (ind.num==30)
pred0mat1 <- matrix(0,nrow(p0.TB),len1);

for (i in 1:nrow(p0.TB) ) {
ix <- seq(j,by=num.cova,length=6);

para <- c(TB0.qnt1[i,j],TB0.qnt2[i,j],CR0.qnt1[i,j],CR0.qnt2[i,j],p0.TB[i,j],p0.zero[i,j])

p <- mixture0(para);
p <- p$mix;
p <- p[2:(ind.num+1)];
p <- p/sum(p);
p <- c(0,cumsum(p));
h <- hist(runif(sum(ffreq0),0,1),breaks=p,plot=F);
pred0mat[i,] <- ffreq0-h$counts;

if (ind.num==30) {
for (jj in 1:len1)  {
pred0mat1[i,jj] <- freq01[jj,j]-sum(h$counts[pred.int[jj,1]:pred.int[jj,2]]);
}
}

}       # end of i loop

#       differences: observed - predicted
obs.pred <- MSoutput(pred0mat,c(0.025,0.50,0.975));
obs.pred <- cbind(ffreq0,ffreq0-obs.pred[,1],obs.pred);

if (ind.num==30)        {
obs.pred1 <- MSoutput(pred0mat1,c(0.025,0.50,0.975));
obs.pred1 <- cbind(freq01[,j],freq01[,j]-obs.pred1[,1],obs.pred1);
}

dimnames(obs.pred) <- list(ind.names,c("obs","pred",paste("diff-",out.str,sep="")));

if (ind.num==30)        {
dimnames(obs.pred1) <- list(ind.names1,c("observed","predicted",out.str));
}
print(round(obs.pred,1))

pred0.low <- which(obs.pred[,5]>0);
pred0.high <- which(obs.pred[,7]<0);
pred0.table[pred0.low,j] <-  -1
pred0.table[pred0.high,j] <- 1

cat("\n");
cat("Predictive failures","\n");
cat(" predictions too small: ",ind.names[pred0.low],"\n");
cat(" predictions too large: ",ind.names[pred0.high],"\n","\n");

if (ind.num==30)        {
print(round(obs.pred1,1));
pred0.low <- which(obs.pred1[,5]>0);
pred0.high <- which(obs.pred1[,7]<0);
cat("\n");
cat("Predictive failures","\n");
cat(" predictions too small: ",ind.names1[pred0.low],"\n");
cat(" predictions too large: ",ind.names1[pred0.high],"\n","\n");
}

}       # end of j loop


############################################################
sink(file=outfilelog,append=T)
 cat(" ...BCG","\n")
sink()
Sys.sleep(1)
############################################################

# BCG
for (j in 1:num.cova)   {
sink();
Sys.sleep(1)
cat("\n","posterior predictive model check: ",paste(groups.labels[j],"(BCG)"));

sink(file=outfile3,append=T);

cat(paste(groups.labels[j],"(BCG)"),"\n");
ffreq1 <- freq1[,j];
pred1mat <- matrix(0,nrow(p1.TB),length(ind));

if (ind.num==30)
pred1mat1 <- matrix(0,nrow(p1.TB),len1);

for (i in 1:nrow(p1.TB) ) {
ix <- seq(j,by=num.cova,length=6);

para <- c(TB1.qnt1[i,j],TB1.qnt2[i,j],CR1.qnt1[i,j],CR1.qnt2[i,j],BCG.qnt1[i,j],BCG.qnt2[i,j],
p1.TB[i,j],p1.zero[i,j],p1.BCG[i,j])

p <- mixture1(para);
p <- p$mix;
p <- p[2:(ind.num+1)];
p <- p/sum(p);
p <- c(0,cumsum(p));
h <- hist(runif(sum(ffreq1),0,1),breaks=p,plot=F);
pred1mat[i,] <- ffreq1-h$counts;

if (ind.num==30) {
for (jj in 1:len1)  {
pred1mat1[i,jj] <- freq11[jj,j]-sum(h$counts[pred.int[jj,1]:pred.int[jj,2]]);
}
}

}       # end of i loop

#       differences: observed - predicted
obs.pred <- MSoutput(pred1mat,c(0.025,0.50,0.975));
obs.pred <- cbind(ffreq1,ffreq1-obs.pred[,1],obs.pred);

if (ind.num==30)        {
obs.pred1 <- MSoutput(pred1mat1,c(0.025,0.50,0.975));
obs.pred1 <- cbind(freq11[,j],freq11[,j]-obs.pred1[,1],obs.pred1);
}

dimnames(obs.pred) <- list(ind.names,c("obs","pred",paste("diff-",out.str,sep="")));

if (ind.num==30)        {
dimnames(obs.pred1) <- list(ind.names1,c("observed","predicted",out.str));
}
print(round(obs.pred,1))

pred1.low <- which(obs.pred[,5]>0);
pred1.high <- which(obs.pred[,7]<0);
pred1.table[pred1.low,j] <-  -1
pred1.table[pred1.high,j] <-  1

cat("\n");
cat("Predictive failures","\n");
cat(" predictions too small: ",ind.names[pred1.low],"\n");
cat(" predictions too large: ",ind.names[pred1.high],"\n","\n");

if (ind.num==30)        {
print(round(obs.pred1,1));
pred1.low <- which(obs.pred1[,5]>0);
pred1.high <- which(obs.pred1[,7]<0);
cat("\n");
cat("Predictive failures","\n");
cat(" predictions too small: ",ind.names1[pred1.low],"\n");
cat(" predictions too large: ",ind.names1[pred1.high],"\n","\n");
}

}       # end of j loop


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Predictive summaries","\n")
sink()
Sys.sleep(1)
############################################################



pred0.table <- matrix(pred0.table,ind.num,num.cova)
pred0.table <- rbind( pred0.table, apply(pred0.table,2,function(x) sum(abs(x)) ) )
pred0.table <- cbind( pred0.table, apply(pred0.table,1,function(x) sum(abs(x)) ) )

pp0 <- matrix(pred0.table[1:ind.num,1:num.cova])

replace(pp0,-1,"-")
replace(pp0,0,"")
replace(pp0,1,"+")

pred0.table[1:ind.num,1:num.cova] <- pp0
pred0.table <- data.frame(pred0.table)


cat("\n","\n")
cat("Non-BCG: summary of predictive checks (+ predictions too large, - predictions too small)")
cat("\n","  ",round(100*pred0.table[ind.num+1,num.cova+1]/ind.num/num.cova,1),"% predictive failures","\n","\n")

dimnames(pred0.table) <- list(c(ind.names,"total"),c(seq(1,num.cova),"total"))
cat( paste(seq(1,num.cova),": ",groups.labels,sep=""),"\n","\n")
print(pred0.table)
cat("\n")



pred1.table <- matrix(pred1.table,ind.num,num.cova)
pred1.table <- rbind( pred1.table, apply(pred1.table,2,function(x) sum(abs(x)) ))
pred1.table <- cbind( pred1.table, apply(pred1.table,1,function(x) sum(abs(x)) ))

pp1 <- matrix(pred1.table[1:ind.num,1:num.cova])

replace(pp1,-1,"-")
replace(pp1,0,"")
replace(pp1,1,"+")

pred1.table[1:ind.num,1:num.cova] <- pp1
pred1.table <- data.frame(pred1.table)

cat("\n","\n")
cat("BCG: summary of predictive checks (+ predictions too large, - predictions too small)")
cat("\n","  ",round(100*pred1.table[ind.num+1,num.cova+1]/ind.num/num.cova,1),"% predictive failures","\n","\n")

dimnames(pred1.table) <- list(c(ind.names,"total"),c(seq(1,num.cova),"total"))
cat( paste(seq(1,num.cova),": ",groups.labels,sep=""),"\n","\n")
print(pred1.table)
cat("\n")

sink();
Sys.sleep(1)
sink(file=outfile1,append=T)

cat("\n","\n")
cat("Non-BCG: summary of predictive checks (+ predictions too large, - predictions too small)")
cat("\n","  ",round(100*pred0.table[ind.num+1,num.cova+1]/ind.num/num.cova,1),"% predictive failures","\n","\n")

dimnames(pred0.table) <- list(c(ind.names,"total"),c(seq(1,num.cova),"total"))
cat( paste(seq(1,num.cova),": ",groups.labels,sep=""),"\n","\n")
print(pred0.table)
cat("\n")


cat("\n","\n")
cat("BCG: summary of predictive checks (+ predictions too large, - predictions too small)")
cat("\n","  ",round(100*pred1.table[ind.num+1,num.cova+1]/ind.num/num.cova,1),"% predictive failures","\n","\n")

dimnames(pred1.table) <- list(c(ind.names,"total"),c(seq(1,num.cova),"total"))
cat( paste(seq(1,num.cova),": ",groups.labels,sep=""),"\n","\n")
print(pred1.table)
cat("\n")

sink()
Sys.sleep(1)

}




if (any(MS.graph))   {

############################################################
sink(file=outfilelog,append=T)
 cat("MS.graph: Creating graphs","\n")
sink()
Sys.sleep(1)
############################################################


if (length(MS.graph)==1) MS.graph <- rep(MS.graph,5)
perpage <- MS.graph.page[1]*MS.graph.page[2];

indind <- c(0,ind.brk,30.5)
indind <- (indind[2:length(indind)]+indind[1:(length(indind)-1)])/2

win.graph()

if (MS.size[3]>1) maintext <- paste("Deviance from sampled values (thinning=",MS.size[3],")",sep="")
else maintext="Deviance from sampled values"

plot( fit$LoglikLogpostSample[,3], -2*fit$LoglikLogpostSample[,2],
type="n",ylab="Deviance",xlab="",main=maintext)

e <- fit$LoglikLogpostSample[,3]<=0
lines( fit$LoglikLogpostSample[e,3], -2*fit$LoglikLogpostSample[e,2],lty=2)
e <- fit$LoglikLogpostSample[,3]>=0
lines( fit$LoglikLogpostSample[e,3], -2*fit$LoglikLogpostSample[e,2],lty=1)
abline(v=0,lty=2)
#text(0-MS.size[2]*0.02,min(fit$LoglikLogpostSample[,2]),"burn-in",adj=1)
#text(0+MS.size[2]*0.02,min(fit$LoglikLogpostSample[,2]),"sample",adj=0)





#       MCMC plots

if (MS.graph[1])  {

############################################################
sink(file=outfilelog,append=T)
 cat(" ...MCMC plots","\n")
sink()
Sys.sleep(1)
############################################################



paras <-
cbind(
fit$Sample[[1]],
fit$Sample[[2]],
fit$Sample[[3]],
fit$Sample[[4]],
fit$Sample[[11]],
fit$Sample[[12]])


for (i in 1:length(para0.names)) {
if (i%%perpage==1 | perpage==1 )      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
plot(paras[,i],type="l",main=para0.names[i],xlab="",ylab="");
}


paras <-
cbind(
fit$Sample[[5]],
fit$Sample[[6]],
fit$Sample[[7]],
fit$Sample[[8]],
fit$Sample[[9]],
fit$Sample[[10]],
fit$Sample[[13]],
fit$Sample[[14]])

for (i in 1:length(para1.names)) {
if (i%%perpage==1 | perpage==1 )      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
plot(paras[,i],type="l",main=para1.names[i],xlab="",ylab="");
}


}


#       SAMPLE HISTOGRAMS

if (MS.graph[2])  {

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Histograms for parameter estimates","\n")
sink()
Sys.sleep(1)
############################################################


paras <-
cbind(
fit$Sample[[1]],
fit$Sample[[2]],
fit$Sample[[3]],
fit$Sample[[4]],
fit$Sample[[11]],
fit$Sample[[12]])

#       SAMPLE HISTOGRAMS
for (i in 1:length(para0.names)) {
if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
hist(paras[,i],type="l",main=para0.names[i],xlab="",ylab="");
}

paras <-
cbind(
fit$Sample[[5]],
fit$Sample[[6]],
fit$Sample[[7]],
fit$Sample[[8]],
fit$Sample[[9]],
fit$Sample[[10]],
fit$Sample[[13]],
fit$Sample[[14]])

#       SAMPLE HISTOGRAMS
for (i in 1:length(para1.names)) {
if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
hist(paras[,i],type="l",main=para1.names[i],xlab="",ylab="");
}

}











#       PROBABILITY of INFECTION

if (MS.graph[3])    {

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Graphs for probability of infection","\n")
sink()
Sys.sleep(1)
############################################################


mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;

#       non-BCG
for (j in 1:num.cova)   {

theta <- mpe0[j,c(1,2,3,4,11,12)]
dist <- mixture0(theta);
mix <- dist$mix;
n <- sum(freq0zero[,j]);
mixobs[,j] <-  dist$mix[2:(ind.num+1)]*n;
p1f1obs[,j] <- dist$p1TB*n;
p2f2obs[,j] <- dist$p2CR*n;
}


for (i in 1:num.cova)   {
if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
plot(indind,p1f1obs[,i]/(p1f1obs[,i]+p2f2obs[,i]),type="l",
xlab="induration",ylab="Pr [ TB | ind ]",
main = paste(groups.labels[i],"(non-BCG)"),
xlim=c(0,20),ylim=c(0,1));
abline(h=0.50,lty=3);
}


mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;
p3f3obs <- mixobs;

for (j in 1:num.cova)   {
theta <- mpe0[j,c(5,6,7,8,9,10,13,14,15)]
dist <- mixture1(theta);
mix <- dist$mix;
n <- sum(freq1zero[,j]);
mixobs[,j] <-  dist$mix[2:(ind.num+1)]*n;
p1f1obs[,j] <- dist$p1TB*n;
p2f2obs[,j] <- dist$p2CR*n;
p3f3obs[,j] <- dist$p3BCG*n;
}

for (i in 1:num.cova)   {
if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
plot(indind,p1f1obs[,i]/(p1f1obs[,i]+p2f2obs[,i]+p3f3obs[,i]),type="l",
xlab="induration",ylab="Pr [ TB | ind ]",
main = paste(groups.labels[i],"(BCG)"),
xlim=c(0,20),ylim=c(0,1))
abline(h=0.50,lty=3);
}

}


if (MS.graph[4])
{

if (num.cova>1)
{

win.graph()
GS <- p0.TB.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="p0.TB",
main="Prevalence of TB infection (non-BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

win.graph()
GS <- p1.TB.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="p1.TB",
main="Prevalence of TB infection (BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

win.graph()
GS <- p0.zero.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="p0.zero",
main="Prevalence of zero induration (non-BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

win.graph()
GS <- p1.zero.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="p1.zero",
main="Prevalence of zero induration (BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}




win.graph()
par(mfrow=c(2,3))

GS <- TB0.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="TB0.qnt1",
main="TB0.qnt1 (non-BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- TB1.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="TB1.qnt1",
main="TB1.qnt1 (BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- BCG.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="BCG.qnt1",
main="BCG.qnt1",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}


GS <- CR0.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="CR0.qnt1",
main="CR0.qnt1 (non-BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- CR1.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="CR1.qnt1",
main="CR1.qnt1 (BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}


win.graph()
par(mfrow=c(2,3))

GS <- TB0.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="TB0.qnt2",
main="TB0.qnt2 (non-BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- TB1.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="TB1.qnt2",
main="TB1.qnt2 (BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- BCG.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="BCG.qnt2",
main="BCG.qnt2",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}


GS <- CR0.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="CR0.qnt2",
main="CR0.qnt2 (non-BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- CR1.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="CR1.qnt2",
main="CR1.qnt2 (BCG)",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

}


}       # end of confidence intervals





#       HISTOGRAMS and FIT

if (MS.graph[5])
{

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Data histograms and fits","\n")
sink()
Sys.sleep(1)
############################################################


mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;

#       non-BCG
for (j in 1:num.cova)   {

theta <- mpe0[j,c(1,2,3,4,11,12)]
dist <- mixture0(theta);
mix <- dist$mix;
n <- sum(freq0zero[,j]);
mixobs[,j] <-  dist$mix[2:(ind.num+1)]*n;
p1f1obs[,j] <- dist$p1TB*n;
p2f2obs[,j] <- dist$p2CR*n;
}

if (length(MS.graph.page)==1)
MS.graph.page <- c(MS.graph.page,MS.graph.page)


# NON-BCG
for (i in 1:num.cova)   {
if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}
if ( max(mixobs[,i]) > max(freq0[,i]) )  {
plot(indind,mixobs[,i],type="l",ylab="n (non-BCG)",
main=groups.labels[i]);
lines(indind,p1f1obs[,i],lty=2);
lines(indind,p2f2obs[,i],lty=2);
points(c(indind,31)-0.5,c(freq0[,i],0),type="s");
}

if ( max(mixobs[,i]) < max(freq0[,i]) )  {
plot(c(indind,31)-0.5,c(freq0[,i],0),type="s",ylab="n (non-BCG)",xlab="induration",
main=groups.labels[i]);
lines(indind,mixobs[,i]);
lines(indind,p1f1obs[,i],lty=2);
lines(indind,p2f2obs[,i],lty=2);
}
}



#       BCG

mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;
p3f3obs <- mixobs;


for (j in 1:num.cova)   {

theta <- mpe0[j,c(5,6,7,8,9,10,13,14,15)]
dist <- mixture1(theta);
mix <- dist$mix;
n <- sum(freq1zero[,j]);
mixobs[,j] <-  dist$mix[2:(ind.num+1)]*n;
p1f1obs[,j] <- dist$p1TB*n;
p2f2obs[,j] <- dist$p2CR*n;
p3f3obs[,j] <- dist$p3BCG*n;
}

if (length(MS.graph.page)==1)
MS.graph.page <- c(MS.graph.page,MS.graph.page)

perpage <- MS.graph.page[1]*MS.graph.page[2];

indind <- c(0,ind.brk,30.5)
indind <- (indind[2:length(indind)]+indind[1:(length(indind)-1)])/2

for (i in 1:num.cova)   {

if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}

if ( max(mixobs[,i]) > max(freq1[,i]) )  {
plot(indind,mixobs[,i],type="l",ylab="n (BCG)",
main=groups.labels[i]);
lines(indind,p1f1obs[,i],lty=2);
lines(indind,p2f2obs[,i],lty=2);
lines(indind,p3f3obs[,i],lty=2);
points(c(indind,31)-0.5,c(freq1[,i],0),type="s");
}

if ( max(mixobs[,i]) < max(freq1[,i]) )  {
plot(c(indind,31)-0.5,c(freq1[,i],0),type="s",ylab="n (BCG)",xlab="induration",
main=groups.labels[i]);
lines(indind,mixobs[,i]);
lines(indind,p1f1obs[,i],lty=2);
lines(indind,p2f2obs[,i],lty=2);
lines(indind,p3f3obs[,i],lty=2);
}

}       # end of histograms and fit

}

}       #  end of MS.graph



Program=paste("Program",Version)

MS.info <- list(
MS.size=MS.size,
MS.block=MS.block,
MS.low=MS.low,
MS.high=MS.high,
MS.adj=MS.adj,
MS.screen=MS.screen,
MS.screen.dig=MS.screen.dig,
MS.names=MS.names,
MS.InitialValues=theta00,
MS.run=MS.run,
MS.results=MS.results,
MS.check=MS.check,
MS.graph=MS.graph,
MS.graph.page=MS.graph.page
)

Data=list(DataNonBCG=freq0all,DataBCG=freq1all)

files=c(infile0,infile1,outfile)
dist <- c(distTB=distTB,distCR=distCR,distBCG=distBCG)

constraints <- list(
TB0.qnt1.diff=TB0.qnt1.diff,
TB0.qnt2.diff=TB0.qnt2.diff,
CR0.qnt1.diff=CR0.qnt1.diff,
CR0.qnt2.diff=CR0.qnt2.diff,
TB1.qnt1.diff=TB1.qnt1.diff,
TB1.qnt2.diff=TB1.qnt2.diff,
CR1.qnt1.diff=CR1.qnt1.diff,
CR1.qnt2.diff=CR1.qnt2.diff,
BCG.qnt1.diff=BCG.qnt1.diff,
BCG.qnt2.diff=BCG.qnt2.diff,
TB01.qnt1.diff=TB01.qnt1.diff,
TB01.qnt2.diff=TB01.qnt2.diff,
CR01.qnt1.diff=CR01.qnt1.diff,
CR01.qnt2.diff=CR01.qnt2.diff,
TB0.qnt1.ratio=TB0.qnt1.ratio,
TB0.qnt2.ratio=TB0.qnt2.ratio,
CR0.qnt1.ratio=CR0.qnt1.ratio,
CR0.qnt2.ratio=CR0.qnt2.ratio,
TB1.qnt1.ratio=TB1.qnt1.ratio,
TB1.qnt2.ratio=TB1.qnt2.ratio,
CR1.qnt1.ratio=CR1.qnt1.ratio,
CR1.qnt2.ratio=CR1.qnt2.ratio,
BCG.qnt1.ratio=BCG.qnt1.ratio,
BCG.qnt2.ratio=BCG.qnt2.ratio,
TB01.qnt1.ratio=TB01.qnt1.ratio,
TB01.qnt2.ratio=TB01.qnt2.ratio,
CR01.qnt1.ratio=CR01.qnt1.ratio,
CR01.qnt2.ratio=CR01.qnt2.ratio)

############################################################
sink(file=outfilelog,append=T)
 cat("Clearing Memory","\n")
sink()
Sys.sleep(1)
############################################################

############################################################
sink(file=outfilelog,append=T)
 cat(date(),"\n","\n")
sink()
Sys.sleep(1)
############################################################


return(
list(
Program=Program,
fit=fit,
pred0.table=pred0.table,
pred1.table=pred1.table,
MSinfo=MS.info,
constraints=constraints,
files=files,
dist=dist,
Data=Data,
p0.TB.summary=p0.TB.summary,
p0.zero.summary=p0.zero.summary,
TB0.qnt1.summary=TB0.qnt1.summary,
TB0.qnt2.summary=TB0.qnt2.summary,
CR0.qnt1.summary=CR0.qnt1.summary,
CR0.qnt2.summary=CR0.qnt2.summary,
p1.TB.summary=p1.TB.summary,
p1.zero.summary=p1.zero.summary,
TB1.qnt1.summary=TB1.qnt1.summary,
TB1.qnt2.summary=TB1.qnt2.summary,
CR1.qnt1.summary=CR1.qnt1.summary,
CR1.qnt2.summary=CR1.qnt2.summary,
BCG.qnt1.summary=BCG.qnt1.summary,
BCG.qnt2.summary=BCG.qnt2.summary,
mpe=mpe0,
LastValues=LastValues,
RndSeedAtStart=RndSeed)
)
}

TBmix <- bcg()



if (!is.na(match("TB0.qnt1.diff",obj))) rm(TB0.qnt1.diff )
if (!is.na(match("TB0.qnt2.diff",obj))) rm(TB0.qnt2.diff )
if (!is.na(match("CR0.qnt1.diff",obj))) rm(CR0.qnt1.diff )
if (!is.na(match("CR0.qnt2.diff",obj))) rm(CR0.qnt2.diff )

if (!is.na(match("TB0.qnt1.ratio",obj))) rm(TB0.qnt1.ratio )
if (!is.na(match("TB0.qnt2.ratio",obj))) rm(TB0.qnt2.ratio )
if (!is.na(match("CR0.qnt1.ratio",obj))) rm(CR0.qnt1.ratio )
if (!is.na(match("CR0.qnt2.ratio",obj))) rm(CR0.qnt2.ratio )

if (!is.na(match("TB1.qnt1.diff",obj))) rm(TB1.qnt1.diff )
if (!is.na(match("TB1.qnt2.diff",obj))) rm(TB1.qnt2.diff )
if (!is.na(match("CR1.qnt1.diff",obj))) rm(CR1.qnt1.diff )
if (!is.na(match("CR1.qnt2.diff",obj))) rm(CR1.qnt2.diff )
if (!is.na(match("BCG.qnt1.diff",obj))) rm(BCG.qnt1.diff )
if (!is.na(match("BCG.qnt2.diff",obj))) rm(BCG.qnt2.diff )

if (!is.na(match("TB1.qnt1.ratio",obj))) rm(TB1.qnt1.ratio )
if (!is.na(match("TB1.qnt2.ratio",obj))) rm(TB1.qnt2.ratio )
if (!is.na(match("CR1.qnt1.ratio",obj))) rm(CR1.qnt1.ratio )
if (!is.na(match("CR1.qnt2.ratio",obj))) rm(CR1.qnt2.ratio )
if (!is.na(match("BCG.qnt1.ratio",obj))) rm(BCG.qnt1.ratio )
if (!is.na(match("BCG.qnt2.ratio",obj))) rm(BCG.qnt2.ratio )

if (!is.na(match("TB01.qnt1.diff",obj))) rm(TB01.qnt1.diff )
if (!is.na(match("TB01.qnt2.diff",obj))) rm(TB01.qnt2.diff )
if (!is.na(match("CR01.qnt1.diff",obj))) rm(CR01.qnt1.diff )
if (!is.na(match("CR01.qnt1.diff",obj))) rm(CR01.qnt2.diff )

if (!is.na(match("TB01.qnt1.ratio",obj))) rm(TB01.qnt1.ratio )
if (!is.na(match("TB01.qnt2.ratio",obj))) rm(TB01.qnt2.ratio )
if (!is.na(match("CR01.qnt1.ratio",obj))) rm(CR01.qnt1.ratio )
if (!is.na(match("CR01.qnt1.ratio",obj))) rm(CR01.qnt2.ratio )

if (!is.na(match("p0.TB.init",obj)))      rm(p0.TB.init)
if (!is.na(match("TB0.qnt1.init",obj)))   rm(TB0.qnt1.init)
if (!is.na(match("TB0.qnt2.init",obj)))   rm(TB0.qnt2.init)
if (!is.na(match("CR0.qnt1.init",obj)))   rm(CR0.qnt1.init)
if (!is.na(match("CR0.qnt2.init",obj)))   rm(CR0.qnt2.init)

if (!is.na(match("p1.TB.init",obj)))      rm(p1.TB.init)
if (!is.na(match("TB1.qnt1.init",obj)))   rm(TB1.qnt1.init)
if (!is.na(match("TB1.qnt2.init",obj)))   rm(TB1.qnt2.init)
if (!is.na(match("CR1.qnt1.init",obj)))   rm(CR1.qnt1.init)
if (!is.na(match("CR1.qnt2.init",obj)))   rm(CR1.qnt2.init)
if (!is.na(match("BCG.qnt1.init",obj)))   rm(BCG.qnt1.init)
if (!is.na(match("BCG.qnt2.init",obj)))   rm(BCG.qnt2.init)

if (!is.na(match("p0.TB.int",obj)))      rm(p0.TB.int)
if (!is.na(match("TB0.qnt1.int",obj)))   rm(TB0.qnt1.int)
if (!is.na(match("TB0.qnt2.int",obj)))   rm(TB0.qnt2.int)
if (!is.na(match("CR0.qnt1.int",obj)))   rm(CR0.qnt1.int)
if (!is.na(match("CR0.qnt2.int",obj)))   rm(CR0.qnt2.int)

if (!is.na(match("p1.TB.int",obj)))      rm(p1.TB.int)
if (!is.na(match("TB1.qnt1.int",obj)))   rm(TB1.qnt1.int)
if (!is.na(match("TB1.qnt2.int",obj)))   rm(TB1.qnt2.int)
if (!is.na(match("CR1.qnt1.int",obj)))   rm(CR1.qnt1.int)
if (!is.na(match("CR1.qnt2.int",obj)))   rm(CR1.qnt2.int)
if (!is.na(match("BCG.qnt1.int",obj)))   rm(BCG.qnt1.int)
if (!is.na(match("BCG.qnt2.int",obj)))   rm(BCG.qnt2.int)

if (!is.na(match("qnt1TB",obj))) rm(qnt1TB)
if (!is.na(match("qnt2TB",obj))) rm(qnt2TB)
if (!is.na(match("qnt1CR",obj))) rm(qnt1CR)
if (!is.na(match("qnt2CR",obj))) rm(qnt2CR)
if (!is.na(match("qnt1BCG",obj))) rm(qnt1BCG)
if (!is.na(match("qnt2BCG",obj))) rm(qnt2BCG)

if (!is.na(match("MS.run",obj))) rm(MS.run)
if (!is.na(match("MS.results",obj))) rm(MS.results)
if (!is.na(match("MS.check",obj))) rm(MS.check)
if (!is.na(match("MS.graph",obj))) rm(MS.graph)
if (!is.na(match("MS.size",obj))) rm(MS.size)
if (!is.na(match("MS.graph.page",obj)))  rm(MS.graph.page)
if (!is.na(match("MS.screen",obj)))  rm(MS.screen)
if (!is.na(match("MS.InitialValues",obj)))  rm(MS.InitialValues)
if (!is.na(match("RndSeed",obj)))  rm(RndSeed)

if (!is.na(match("ind.names",obj))) rm(ind.names)
if (!is.na(match("group.names",obj))) rm(group.names)
if (!is.na(match("outfile",obj))) rm(outfile)
if (!is.na(match("infile0",obj))) rm(infile0)
if (!is.na(match("infile1",obj))) rm(infile1)
if (!is.na(match("freq0.column",obj))) rm(freq0.column)
if (!is.na(match("freq1.column",obj))) rm(freq1.column)
if (!is.na(match("distTB",obj))) rm(distTB)
if (!is.na(match("distCR",obj))) rm(distCR)
if (!is.na(match("distBCG",obj))) rm(distBCG)
if (!is.na(match("header",obj))) rm(header)


